/*
 * a1-types.h
 * Copyright (C) 2006-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_a1_TYPES_H__
#define __LIBx1f4_a1_TYPES_H__

#include <config.h>

#include <c1.h>
#include <e4.h>
#include <e4jack.h>
#include <frame.h>
#include <integral_q.h>
#include <it.h>
#include <ln.h>
#include <track.h>
#include <trans.h>

#define HERE_A1_C_WIND			void *

typedef struct a1_back_type {
    const char *line;
    struct {
	int (*rule)(void *, unsigned, unsigned);
	void *data;
    } hint;
    union {
	struct {
	    x1f4_c1record_type c1record;
	} c_syntax;
	struct {
	    unsigned size;
	} flattype;
	struct {
	    x1f4_ejrecord_type ejrecord;
	} higherof;
	struct {
	    unsigned size;
	} lesslink;
	struct {
	    int type;
	} maintype;
	struct {
	    int type;
	} misslead;
	struct {
	    const char *fast;
	    union {
		struct {
		    unsigned name;
		} namecall;
		struct {
		    int slip, type;
		    unsigned name;
		} nametype;
		struct {
		    int slip, type;
		} sliptype;
	    } data;
	    unsigned code, size;
	} missline;
	struct {
	    unsigned size;
	} misslink;
	struct {
	    unsigned size;
	} missname;
	struct {
	    unsigned size;
	} missturn;
	struct {
	    unsigned size;
	} misstype;
	struct {
	    const char *fast;
	    unsigned size;
	} postfeat;
	struct {
	    unsigned size;
	} reserved;
	struct {
	    unsigned size;
	} resolved;
	struct {
	    int type;
	} sideline;
	struct {
	    unsigned size;
	} xdefined;
    } data;
    unsigned code, pick;
} a1_back_type;

typedef struct a1_context_type {
    struct x1f4_track_type call_e;
    struct {
	struct x1f4_eelookup_type *eelookup_data;
    } trap_e;
    struct {
	int (*fix)(const void *, const struct x1f4_function_type *, void **),
	    (*get)(const char *, unsigned, const void *,
		   const struct x1f4_function_type **);
	const void *context;
    } last_0;
    struct {
	const void *context;
    } ever_0;
    struct x1f4_iturn_type trap_0;
    struct x1f4_composer_type turn_0;
    struct {
	void *back;
    } link_e;
    struct {
	void *fine;
    } link_i;
    struct x1f4_trans_type link_m;
    struct {
	struct x1f4_linetrap_type *linetrap_data;
	unsigned bits;
    } link_y;
    struct {
	int (*link)(void *, void **, unsigned),
	    (*pick)(void *, void ***, int (*)(void *), unsigned, void **);
	void *data;
    } near_f;
    struct {
	const void *miss;
    } user_w;
    struct {
	const void *miss;
    } user_z;
    struct {
	const void *miss;
    } verb_a;
    struct {
	void *fine;
    } verb_e;
    struct {
	struct x1f4_linetext_type **linetext;
	struct x1f4_variable_type *variable_data;
    } verb_b;
    struct {
	const char **argv;
	unsigned argc;
    } verb_c;
    struct x1f4_frame_type were_f;
    struct x1f4_trans_type were_w;
    struct x1f4_scope_type were_x;
    const struct x1f4_operator_type **operator1s, **operator2s;
} a1_context_type;

typedef struct a1_fast_type {
    struct a1_fast_type *fast_data;
    struct x1f4_dispatch_type *dispatch_data;
    void *dispatched_output;
} a1_fast_type;

typedef struct a1_flat_type {
    struct a1_flat_type *flat_data;
} a1_flat_type;

/*
 * the _v_ is needed for nothing else but delaying destruction of alien (i.e.
 * other programs) functions when "pushing".
 */
typedef struct a1_grid_type {
    struct {
	int (*pick)(void *, void ***, int (*)(void *), unsigned, void **);
	void *data;
    } v;
    struct trans_type trans;
    unsigned call;
} a1_grid_type;

typedef struct a1_lead_type {
    union {
	X1f4_E4_C_MODE mode;
	X1f4_E4_C_REAL real;
	X1f4_E4_C_TEXT text;
	X1f4_E4_C_USER user;
	HERE_A1_C_WIND wind;
    } data;
} a1_lead_type;

typedef struct a1_line_type {
    struct a1_line_type *line_data;
    void *node;
} a1_line_type;

typedef struct a1_link_type {
    int (*line)(void **, void **, struct a1_link_type *, void *),
	(*lose)(void *, struct a1_link_type *, void *);
    struct x1f4_datatext_type datatext;
    struct x1f4_variable_type variable;
} a1_link_type;

typedef struct a1_lock_type {
    struct a1_fast_type *fast_head, *fast_tail;
    struct a1_lock_type *lock_data;
    struct a1_node_type *fail_node, *node_data;
    struct x1f4_dispatch_type *dispatch_data;
    const struct x1f4_function_type *function_data, *function_lead;
    void *dispatched_output, *head, *lead, *output, *port, *slip, *tail;
} a1_lock_type;

/*
 * _origin_ is needed for the sole purpose of detecting alien functions and
 * stopping their self destruction.
 */
typedef struct a1_node_type {
    struct a1_grid_type *grid_data;
    struct a1_line_type *line_data;
    struct a1_link_type *link_data;
    struct a1_node_type **fast, *node_data;
    struct x1f4_leadnode_type leadnode;
    struct x1f4_linetext_type linetext;
    unsigned extension, offset, post, side;
    void *aime;
} a1_node_type;

typedef struct a1_pass_type {
    int (*back)(void *, const struct x1f4_function_type *),
	(*fast)(void *, const struct x1f4_function_type *),
	(*lose)(void *, const struct x1f4_function_type *);
    void *text;
} a1_pass_type;

typedef struct a1_pipe_type {
    struct a1_pass_type pass;
    struct a1_pipe_type *back_pipe, *pipe_long;
} a1_pipe_type;

struct a1_shuffle_type;

typedef struct a1_instant_type {
    struct {
	const struct x1f4_function_type *function_data;
	void *lead;
    } s;
    struct x1f4_trans_type m;
    struct x1f4_frame_type v;
    struct a1_node_type *fail_node;
    struct a1_shuffle_type *shuffle_data;
} a1_instant_type;

/*
 * the _main_ function is pointed to by _slip_node_
 * maybe this attribute should be removed and have the _main_ function
 * pointed to by the first in the _node_data_ linked list
 */
/*
 * _instant_data_ should not be here.  needed by _seek_shuffle only.
 */
typedef struct a1_shuffle_type {
    struct a1_grid_type *grid_data;
    struct a1_instant_type *instant_data;
    struct x1f4_trans_type m;
    struct x1f4_iturn_type p;
    struct x1f4_composer_type q;
    struct {
	const struct x1f4_datatype_type *datatype_data, *sidetype_data;
    } u;
    struct x1f4_frame_type v;
    struct x1f4_track_type x;
    struct x1f4_scope_type z;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } y;
    struct a1_flat_type *flat_data;
    struct a1_node_type *node_data, *slip_node;
    struct a1_pipe_type *back_pipe, *pipe_long;
    void *fine, *side;
} a1_shuffle_type;

typedef struct a1_trans18_type {
    struct {
	void *used;
    } i;
    struct {
	struct a1_instant_type instant;
    } r;
    struct {
	int (*call)(void *), *note;
	void *back;
    } z;
} a1_trans18_type;

typedef struct a1_trans19_type {
    struct {
	int sample;
	void *head, *lock, *port, *slip, *tail;
    } e;
    struct {
	void *link, *miss, *used;
    } i;
    struct {
	struct a1_instant_type instant;
    } r;
    struct {
	struct x1f4_transfer_type transfer;
    } x;
} a1_trans19_type;

#endif				/* __LIBx1f4_a1_TYPES_H__ */
