/*
 * bqfs.j.c
 * Copyright (C) 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <bqfx-config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <bqfx-bits.h>
#include <bqfx-defs.h>
#include <bqfx-inter.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>
#include <kfs.h>
#include <kls.h>

#define false(e)			0
#define true(e)				1

#define beta(___s, ___n) \
    (___s) = (char *) (___s) - ((___n) << 1)

#define dana(___s, ___n) \
    (___s) = (char *) (___s) + ((___n) << 1)

#define posh(___s, ___n) \
    (___s) = (char *) (___s) - (___n)

#define road(___s, ___n) \
    ((char *) (___s) + ((___n) << 1))

#define rule(___s, ___n, ___t) \
    ((char *) (___s) + (___n) * (___t))

#define sans(___s, ___n) \
    ((char *) (___s) + (___n))

#define skip(___s, ___n) \
    (___s) = (char *) (___s) + (___n)

#undef Q
#define Q(e)				((integral_q) (e))

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

#define tank(a, b, c, d, e, f) \
    _libx1f4l2_tank_bqfset((a), (b), (c), (e), (f))

static int land(void *, void *, void *, unsigned, unsigned, unsigned *,
		unsigned, unsigned, int, void **);
static int last(void *, void *, void *, unsigned, void *, unsigned, unsigned *,
		unsigned, unsigned, void **);
static int lend(void *, void *, unsigned *, unsigned, void *, void *, void *,
		unsigned);
static int lest(void *, void *, unsigned, unsigned, void *, void *, unsigned,
		void *);
static int lift(void *, void *, unsigned, void **);
static int near(void *, void *, integral_q, unsigned, unsigned, void *, void *,
		unsigned *, unsigned, unsigned);
static int norm(void *, unsigned, unsigned);
static int over(void *, void *, integral_q, unsigned, unsigned, void *, void *,
		unsigned, unsigned, unsigned);
static int push(void *, void *, unsigned, unsigned);
static int quit(void *, void *, integral_q, unsigned, unsigned);
static int reel(void *, void *, unsigned, unsigned *, void *, void *);
static int roll(void *, void *, unsigned, unsigned, void *, void *, void *);
static int rush(void *, void *, unsigned, unsigned);
static int rift(void *, void *, unsigned, void **);
static int sand(void *, void *, unsigned, unsigned, void *, unsigned, unsigned,
		unsigned);
static int sort(void *, void *, unsigned, unsigned, void *, unsigned,
		unsigned, void **, void *);
static int toll(void *, void *, unsigned, unsigned, void *);

static int
land(void *bqfset, void *data, void *club, unsigned fold, unsigned call,
     unsigned *slip, unsigned just, unsigned size, int ardv, void **iran)
{
    int (*move)(void *, void *, unsigned);
    integral_q news;
    unsigned i = integral_q_last, suit;
    void *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    suit = just ^ size ? 2 : 1;

    node = rule(club, size, integral_q_bits);
    pull = ZD(club, size) + integral_q_last;

    while (1 < fold) {
	if (news & Q(1) << i) {
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    *iran = *pull--;
	    iran -= suit;
	    *iran = *pull--;
	    iran -= suit;
	    fold -= 2;
	} else {
	    beta(node, size);
	    move(data, node, 1);
	    *iran = *pull;
	    iran -= suit;
	    pull -= 2;
	    fold--;
	}
	{
	    posh(data, just);
	}

	i -= 2;
    }
    if (fold == 1) {
	if (ardv) {
	    if (news & Q(1) << i) {
		beta(node, size);
		move(data, node, 2);
		*iran-- = *pull--;
		*iran = *pull--;

		fold = 0;

		i -= 2;
	    } else {
		beta(node, size);
		move(sans(data, size), node, 1);
		*iran-- = *pull;
		pull -= 2;

		i -= 2;

		if (news & Q(1) << i) {
		    posh(node, size);
		    move(data, node, 1);
		    *iran = *pull;
		} else {
		    beta(node, size);
		    move(data, node, 1);
		    *iran = *pull;
		}
	    }
	} else {
	    if (news & Q(1) << i) {
		posh(node, size);
	    } else {
		beta(node, size);
	    }
	    {
		move(data, node, 1);
		*iran = *pull;
	    }
	}
    } else {
	if (ardv) {
	    dana(data, size);
	    move(sans(data, size), data, 1);
	    iran++;
	    fold = 1;
	    if (news & Q(1) << i) {
		posh(node, size);
	    } else {
		beta(node, size);
	    }
	    {
		move(data, node, 1);
		*iran = *pull;
	    }
	}
    }

    if (call < half_integral_q_bits) {
	*slip = i << 1 | fold;
    } else {
	tank(bqfset, club, i << 1 | fold, club, size << 1, size);
    }

    return 0;
}


static int
last(void *bqfset, void *data, void *tier, unsigned fold, void *turn,
     unsigned call, unsigned *slip, unsigned just, unsigned size, void **iran)
{
    int (*move)(void *, void *, unsigned);
    integral_q news;
    unsigned i = 3, suit;
    void *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    suit = just ^ size ? 2 : 1;

    pull = ZD(tier, size);

    node = tier;

    if (1) {
	if (news & Q(1) << 1) {
	    skip(node, size);
	    if (fold) {
		skip(data, just);
		move(data, node, 1);
		*iran = *pull++;
		iran += suit;
		*iran = *pull;
		iran += suit;
		fold--;
	    } else {
		if (turn) {
		    move(turn, node, 1);
		    turn = NULL;
		} else {
		    i = 1;
		    if (true(call < half_integral_q_bits)) {
			*slip = i << 1 | 1;
		    } else {
			/* ... */
		    }
		}
		*iran = *pull;
	    }
	    skip(node, size);
	} else {
	    dana(node, size);
	    *iran = *++pull;
	    iran += suit;
	}
    }
    while (1 < fold) {
	skip(data, just);
	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(data, just);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	    *iran = *++pull;
	    iran += suit;
	    *iran = *++pull;
	    iran += suit;
	    fold -= 2;
	} else {
	    move(data, node, 1);
	    dana(node, size);
	    pull += 2;
	    *iran = *pull;
	    iran += suit;
	    fold--;
	}

	i += 2;
    }
    if (fold) {
	skip(data, just);
	move(data, node, 1);
	if (news & Q(1) << i) {
	    *iran = *++pull;
	} else {
	    pull += 2;
	    *iran = *pull;
	}
    }

    if (call < half_integral_q_bits) {
	if (1 < i) {
	    *slip = i << 1 | fold;
	}
    } else {
	data = tier;

	iran = ZD(data, size);

	if (fold) {
	    if (news & Q(1) << i) {
		skip(node, size);
		if (true(turn)) {
		    move(turn, node, 1);
		    turn = NULL;
		} else {
		    /* ... */
		}
		skip(node, size);
		*++iran = *++pull;
	    } else {
		dana(node, size);
	    }
	} else {
	    i -= 2;
	}

	if (turn) {
	    i += 2;

	    if (news & Q(1) << i) {
		move(turn, node, 1);
		skip(node, size);
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
		*++iran = *++pull;
		iran += 2;
		*iran = *++pull;
	    } else {
		move(turn, node, 1);
		dana(node, size);
		pull += 2;
		*++iran = *pull;
	    }
	} else {
	    if (fold) {
	    } else {
		iran++;
		pull++;
	    }
	}

	/*
	 * if -turn-, it may be that no data needs transferred.
	 */
	while (node != road(data, size)) {
	    i += 2;

	    if (news & Q(1) << i) {
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
		dana(data, size);
		move(data, node, 1);
		skip(node, size);
		iran += 2;
		*iran = *++pull;
		iran += 2;
		*iran = *++pull;
	    } else {
		dana(data, size);
		move(data, node, 1);
		dana(node, size);
		iran += 2;
		pull += 2;
		*iran = *pull;
	    }
	}

	*((integral_q *) tier) = news & ~Q(1) << i | Lx55555555;
    }

    return 0;
}


static int
lend(void *bqfset, void *club, unsigned *i, unsigned fold, void *side,
     void *turn, void *file, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    unsigned ever, mark;
    void *data;

    move = bqfset(bqfset)->link_f.move;

    ever = integral_q_line - fold;

    mark = integral_q_bits - 1 - (fold << 1);

    if (~*i) {
	sand(bqfset, club, *i, fold, club, integral_q_bits, size << 1, size);

	data = rule(club, size, mark - 1);
    } else {
	data = rule(side, size, mark - 1);

	last(bqfset, data, club, fold, NULL, 0, i, size << 1, size,
	     ZD(side, size) + mark);

	club = side;
    }

    move(data, turn, 1);

    /*
     * may need no transfer to -club- at this time, in which case -land- is
     * only called to retrieve the separator data.
     */

    land(bqfset, beta(data, size), file, ever, integral_q_last, NULL,
	 size << 1, size, 1, ZD(club, size) + mark - 2);

    move(turn, club, 1);

    *((integral_q *) club) = Lx55555555 | 2;

    return 0;
}


static int
lest(void *bqfset, void *tier, unsigned i, unsigned fold, void *tune,
     void *file, unsigned size, void *turn)
{
    int (*move)(void *, void *, unsigned);
    unsigned deck;
    void *data;

    move = bqfset(bqfset)->link_f.move;

    /*
     * -fold- had been tested less than integral_q_line.
     */

    deck = integral_q_line - fold;

    sort(bqfset, tier, i, fold, tier, size << 1, size, ZD(tier, size) + 1,
	 turn);

    data = rule(tier, size, fold << 1);

    move(data, tune, 1);

    last(bqfset, data, file, deck, tune, integral_q_last, NULL, size << 1,
	 size, ZD(tier, size) + 1 + (fold << 1));

    *((integral_q *) tier) = Lx55555555;

    return 0;
}


static int
lift(void *bqfset, void *data, unsigned size, void **plus)
{
    int status;

    do {
	status = bqfset(bqfset)->link_m.link
	    (bqfset(bqfset)->link_m.data, plus,
	     integral_q_bits * (SIZEOF_integral_q + size));
	if (status) {
	    status = LINK_ERROR;
	} else {
	    MD(data, size)[0] = *plus;
	}
    } while (0);

    return status;
}


static int
near(void *bqdata, void *data, integral_q news, unsigned pass, unsigned mind,
     void *turn, void *node, unsigned *i, unsigned deck, unsigned rate)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned even, size;
    void *file;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    file = MD(data, size)[pass - 1];

    news = *(integral_q *) file;

    bq_q_bitcount(even, ~news);

    if (pass == 1) {
	void *tune;

	even = integral_q_bits - even;

	tune = sans(turn, size);

	move(tune, sans(data, size), 1);

	status = norm(bqdata, rate, even + deck);
	if (status) {
	} else {
	    void **iran;

	    data = bqfset(bqdata)->link_a.fpnews.node;

	    quit(bqdata, file, news, 0, size);

	    bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, file);

	    iran = MD(data, size);

	    data = rule(data, even, size);

	    move(data, tune, 1);

	    last(bqdata, data, node, deck, NULL, 0, i, size, size,
		 iran + even);

	    if (mind) {
	    } else {
		bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, node);
	    }
	}
    } else {
	if (even < deck + 1) {
	    void *side;

	    if (mind) {
		status = rift(bqdata, data, size, &side);
		if (status) {
		} else {
		    *i = ~0;
		}
	    } else {
		side = NULL;
	    }

	    lend(bqdata, node, i, deck, side, rule(data, size, pass), file,
		 size);
	} else {
	    if (mind) {
		*i = ~0;
	    }

	    reel(bqdata, file, deck, i, rule(data, size, pass), node);

	    *(integral_q *) data = pass - 1;

	    if (mind) {
	    } else {
		bqfset(bqdata)->link_m.free(bqfset(bqdata)->link_m.data, node);
	    }
	}
    }

    return status;
}


static int
norm(void *bqfset, unsigned rate, unsigned fold)
{
    int status = 0;
    void *node;

    node = bqfset(bqfset)->link_a.fpnews.node;

#if __MICRO_FSETS__
    do {
	if (bqfset(bqfset)->link_a.fpnews.link_v.rate) {
	} else {
	    unsigned size;

	    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

	    status = bqfset(bqfset)->link_m.mode
		(bqfset(bqfset)->link_m.data, &node,
		 integral_q_bits_and_half * (size + SIZEOF_integral_q));
	    if (status) {
		status = MODE_ERROR;
		break;
	    } else {
		bqfset(bqfset)->link_a.fpnews.node = node;
	    }
	}
#endif				/* __MICRO_FSETS__ */

	*((integral_q *) node) = fold;

	bqfset(bqfset)->link_a.fpnews.link_v.rate = rate;
#if __MICRO_FSETS__
    } while (0);
#endif				/* __MICRO_FSETS__ */

    return status;
}


static int
over(void *bqtext, void *text, integral_q news, unsigned pass11, unsigned mind,
     void *turn, void *node, unsigned i, unsigned ever, unsigned rate)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned deck, size;
    void *file;

    move = bqfset(bqtext)->link_f.move;

    size = bqfset(bqtext)->link_a.fpnews.link_v.size;

    file = MD(text, size)[1];

    news = *(integral_q *) file;

    bq_q_bitcount(deck, ~news);

    if (mind == 1) {
	deck = integral_q_bits - deck;

	move(turn, sans(text, size), 1);

	status = norm(bqtext, rate, deck + ever - 1);
	if (status) {
	} else {
	    text = bqfset(bqtext)->link_a.fpnews.node;

	    quit(bqtext, file, news, ever, size);

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, file);

	    move(rule(text, ever, size), turn, 1);

	    sort(bqtext, node, i, ever, text, size, size, MD(text, size),
		 turn);

	    bqfset(bqtext)->link_a.fpnews.node = text;

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, node);
	}
    } else {
	if (deck < ever + 0) {
	    lest(bqtext, node, i, ever, sans(text, size), file, size, turn);
	} else {
	    void **iran;

	    roll(bqtext, file, ever, i, sans(text, size), node, turn);

	    iran = MD(text, size);

	    memmove(iran, iran + 1, mind * sizeof(void *));

	    mind--;

	    *(integral_q *) text = mind;

	    skip(text, size);

	    move(text, sans(text, size), mind);

	    bqfset(bqtext)->link_m.free(bqfset(bqtext)->link_m.data, node);
	}
    }

    return status;
}


static int
push(void *bqfset, void *node, unsigned rate, unsigned size)
{
    int status;
    integral_q news;
    unsigned fold;

    news = *(integral_q *) node;
    bq_q_bitcount(fold, news ^ 1);

    status = norm(bqfset, rate, fold);
    if (status) {
    } else {
	quit(bqfset, node, news, 0, size);

	bqfset(bqfset)->link_m.free(bqfset(bqfset)->link_m.data, node);
    }

    return status;
}


static int
quit(void *bqfset, void *node, integral_q news, unsigned call, unsigned size)
{
    int (*move)(void *, void *, unsigned);
    unsigned i = 3;
    void *data, **iran, **pull;

    data = bqfset(bqfset)->link_a.fpnews.node;

    iran = MD(data, size) + call;
    pull = ZD(node, size);

    data = rule(data, size, call);

    move = bqfset(bqfset)->link_f.move;

    if (1) {
	if (news & Q(1) << 1) {
	    skip(data, size);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	    *iran++ = *pull++;
	    *iran++ = *pull;
	} else {
	    dana(node, size);
	    *iran++ = *++pull;
	}
    }
    do {
	skip(data, size);
	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(data, size);
	    skip(node, size);
	    move(data, node, 1);
	    skip(node, size);
	    *iran++ = *++pull;
	    *iran++ = *++pull;
	} else {
	    move(data, node, 1);
	    dana(node, size);
	    pull += 2;
	    *iran++ = *pull;
	}

	i += 2;
    } while (i < integral_q_bits);

    return 0;
}


static int
reel(void *bqfset, void *club, unsigned pick, unsigned *i, void *head,
     void *file)
{
    int (*move)(void *, void *, unsigned);
    integral_q news, seek;
    unsigned j, late, size;
    void **data, *fast, *side, **text;

    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

    move = bqfset(bqfset)->link_f.move;

    /*
     * stuffing pick + 1:
     */
    late = pick;
    l2_q_jls(j, ~*(integral_q *) club, late);
    j ^= 63;
    late = j;

    news = *(integral_q *) club;

    /*
     * -j- can be integral_q_last!
     */

    fast = rule(club, size, j);
    side = fast;
    data = ZD(club, size) + j;
    text = data;
    seek = news >> j;
    if (true(j & 1)) {
	if (false(news & Q(1) << j)) {
	} else {
	    skip(side, size);
	    *(data - 1) = *text;
	    text++;
	}

	j++;
	seek >>= 1;
    }
    for (; seek; seek >>= 2) {
	if (seek & Q(2)) {
	    move(fast, side, 1);
	    skip(fast, size);
	    skip(side, size);
	    move(fast, side, 1);
	    skip(fast, size);
	    skip(side, size);
	    *data++ = *text++;
	    *data++ = *text++;
	} else {
	    move(fast, side, 1);
	    skip(fast, size);
	    dana(side, size);
	    *data++ = *(text + 1);
	    text += 2;
	}
    }

    move(fast, head, 1);

    *(integral_q *) club = news | (~Q(0) << late);

    if (~*i) {
	sand(bqfset, file, *i, pick, club, integral_q_bits, size, size);
    } else {
	last(bqfset, fast, file, pick, NULL, 0, i, size, size, data);
    }

    return 0;
}


static int
roll(void *bqfset, void *tier, unsigned pick, unsigned i, void *past,
     void *file, void *turn)
{
    int (*move)(void *, void *, unsigned);
    integral_q news, seek;
    unsigned j, late, size;
    void **data, *fast, *side, **text;

    size = bqfset(bqfset)->link_a.fpnews.link_v.size;

    move = bqfset(bqfset)->link_f.move;

    /*
     * stuffing pick - 1 + 1:
     */
    late = pick;
    l2_q_jfs(j, ~(*(integral_q *) tier & ~1), late);
    late = j;

    news = *(integral_q *) tier;

    fast = rule(tier, size, j + 1);
    side = fast;
    text = ZD(tier, size) + j;
    data = text + 1;
    seek = (news & ~Q(3)) << (integral_q_last - j);
    for (; seek; seek <<= 2) {
	if (seek & Q(1) << integral_q_last) {
	    beta(fast, size);
	    beta(side, size);
	    move(fast, side, 2);
	    *--data = *text--;
	    *--data = *text--;
	} else {
	    posh(fast, size);
	    beta(side, size);
	    move(fast, side, 1);
	    *--data = *text;
	    text -= 2;
	}
    }
    {
	if (news & 2) {
	    posh(fast, size);
	    posh(side, size);
	    move(fast, side, 1);
	    *--data = *text--;
	    *--data = *text;
	} else {
	    *--data = *text;
	}
    }

    posh(fast, size);
    move(fast, past, 1);

    *(integral_q *) tier = news | ((Q(2) << late) - 1);

    sort(bqfset, file, i, pick, tier, size, size, NULL, turn);

    return 0;
}


static int
rush(void *bqfset, void *node, unsigned rate, unsigned size)
{
    int status;

    status = push(bqfset, node, rate, size);
    if (status) {
    } else {
	bqfset(bqfset)->link_a.fpnews.link_v.rate = rate;
    }

    return status;
}


static int
rift(void *bqfset, void *data, unsigned size, void **plus)
{
    int status;

    do {
	unsigned copy;

	copy = *(integral_q *) data;

	status = bqfset(bqfset)->link_m.link
	    (bqfset(bqfset)->link_m.data, plus,
	     integral_q_bits * (SIZEOF_integral_q + size));
	if (status) {
	    status = LINK_ERROR;
	} else {
	    MD(data, size)[copy] = *plus;
	}
    } while (0);

    return status;
}


static int
sand(void *bqfset, void *club, unsigned i, unsigned fold, void *data,
     unsigned back, unsigned just, unsigned size)
{
    int flip, (*move)(void *, void *, unsigned), suit;
    integral_q news;
    void **iran, *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    suit = just ^ size ? 2 : 1;

    flip = i & 1;

    i >>= 1;

    node = rule(club, size, i + 1);

    pull = ZD(club, size) + i;

    iran = ZD(data, size) + back - 1;

    data = rule(data, size, back);

    if (flip) {
	beta(node, size);
	if (news & Q(1) << i) {
	    if (i == 1) {
		/*
		 * worst special case?
		 */
		news = 0;
	    } else {
		posh(data, just);
		move(data, node, 1);
		*iran = *--pull;
		iran -= suit;
		fold--;
	    }
	    pull--;
	} else {
	    pull -= 2;
	}
    } else {
	i += 2;
    }

    fold -= news >> 1 & 1;

    while (fold) {
	i -= 2;
	if (news & Q(1) << i) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    *iran = *pull--;
	    iran -= suit;
	    *iran = *pull--;
	    iran -= suit;
	    fold -= 2;
	} else {
	    posh(data, just);
	    beta(node, size);
	    move(data, node, 1);
	    *iran = *pull;
	    iran -= suit;
	    pull -= 2;
	    fold--;
	}
    }
    if (news & 2) {
	if (2) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    *iran = *pull--;
	    iran -= suit;
	    *iran = *pull;
	} else {
	}
    } else {
	*iran = *pull;
    }

    return 0;
}


static int
sort(void *bqfset, void *tier, unsigned i, unsigned fold, void *data,
     unsigned just, unsigned size, void **iran, void *turn)
{
    int flip, (*move)(void *, void *, unsigned), suit;
    integral_q news;
    void *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    flip = i & 1;

    i >>= 1;

    suit = just ^ size ? 2 : 1;

    node = rule(tier, size, i - 1);

    pull = ZD(tier, size) + i;

    if (iran) {
    } else {
	iran = ZD(data, size);
    }

    if (flip) {
	if (news & Q(1) << i) {
	    skip(node, size);
	    move(turn, node, 1);
	    turn = 0;
	    skip(node, size);
	    *iran = *pull++;
	    iran += suit;
	    fold--;
	} else {
	    dana(node, size);
	    pull += 2;
	    i += 2;
	}
    }
    if (turn) {
	if (news & Q(1) << i) {
	    move(turn, node, 1);
	    skip(node, size);
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    *iran = *(pull - 1);
	    iran += suit;
	    *iran = *pull++;
	    iran += suit;
	    fold -= 2;
	} else {
	    move(turn, node, 1);
	    dana(node, size);
	    *iran = *pull++;
	    iran += suit;
	    fold--;
	}
    }

    pull--;

    while (fold) {
	i += 2;

	if (news & Q(1) << i) {
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    skip(data, just);
	    move(data, node, 1);
	    skip(node, size);
	    *iran = *++pull;
	    iran += suit;
	    *iran = *++pull;
	    iran += suit;
	    fold -= 2;
	} else {
	    skip(data, just);
	    move(data, node, 1);
	    dana(node, size);
	    fold--;
	    pull += 2;
	    *iran = *pull;
	    iran += suit;
	}
    }

    return 0;
}


static int
toll(void *bqfset, void *tier, unsigned i, unsigned size, void *turn)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *data, **iran, *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) tier;

    flip = i & 1;

    i >>= 1;

    iran = ZD(tier, size);

    node = rule(tier, size, i - 1);

    pull = ZD(tier, size) + i;

    data = tier;

    if (flip) {
	if (news & Q(1) << i) {
	    skip(node, size);
	    move(turn, node, 1);
	    turn = NULL;
	    skip(node, size);
	    *++iran = *pull;
	} else {
	    dana(node, size);
	    pull += 2;
	    i += 2;
	}
    }
    if (turn) {
	if (news & Q(1) << i) {
	    move(turn, node, 1);
	    skip(node, size);
	    dana(data, size);
	    move(data, node, 1);
	    skip(node, size);
	    *++iran = *(pull - 1);
	    iran += 2;
	    *iran = *pull;
	} else {
	    move(turn, node, 1);
	    dana(node, size);
	    *++iran = *pull;
	}
    }

    dana(data, size);

    while (node != data) {
	i += 2;

	if (news & Q(1) << i) {
	    move(data, node, 1);
	    skip(node, size);
	    dana(data, size);
	    move(data, node, 1);
	    skip(node, size);
	    dana(data, size);
	    iran += 2;
	    *iran = *++pull;
	    iran += 2;
	    *iran = *++pull;
	} else {
	    move(data, node, 1);
	    dana(node, size);
	    dana(data, size);
	    iran += 2;
	    pull += 2;
	    *iran = *pull;
	}
    }

    *(integral_q *) tier = Lx55555555 | news & ~Q(1) << i;

    return 0;
}


int
_libx1f4l2_bleu_bqfset(void *bqdata, void *lock, unsigned rate, void *node,
		       void *aime, unsigned copy, int catch, void *turn,
		       void *bqtext)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned size;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    do {
	integral_q news, miss;

	news = *(integral_q *) node;

	miss = news & ~Q(0) << copy << catch;
	if (miss) {
	    news ^= miss | Q(1);
	    if (news | 1) {
		unsigned deck, ever;

		bq_q_bitcount(deck, news);
		bq_q_bitcount(ever, miss);
		if (1) {
		    if (0) {
		    } else {
			unsigned i;
			void *text;

			ever--;

			status = norm(bqtext, rate, ever);
			if (status) {
			    break;
			} else {
			    text = bqfset(bqtext)->link_a.fpnews.node;

			    land(bqtext, rule(text, size, ever), node,
				 ever + 1, deck, &i, size, size, 0,
				 MD(text, size) + ever);

			    move(turn, text, 1);

			    if (ever == 0) {
				if (rate ^ 1) {
				    status = rush
					(bqtext, *MD(text, size), rate - 1,
					 size);
				} else {
				    status = _libx1f4l2_mood_bqfset
					(bqtext, *MD(text, size), size);
				    if (status) {
					break;
				    }
				}
			    } else {
				*(integral_q *) text = ever;
			    }
			}
			if (deck < half_integral_q_bits) {
			    unsigned jack, left;
			    void *file;

			    miss = *(integral_q *) lock;

			    jack = miss >> integral_q_last;
			    left = integral_q_bits - 3 + jack;

			    file = ZD(lock, size)[left];

			    news = *(integral_q *) file;

			    bq_q_bitcount(ever, ~news);

			    if (0) {
			    } else {
				left++;
				if (ever < deck + 1) {
				    lend(bqdata, node, &i, deck, NULL,
					 rule(lock, size, left), file, size);
				} else {
				    reel(bqdata, file, deck, &i,
					 rule(lock, size, left), node);

				    if (jack) {
					void **iran;

					iran = ZD(lock, size)
					    + integral_q_last;
					*iran = *(iran - 1);
					*(integral_q *) lock =
					    miss ^ Q(1) << integral_q_last;
				    } else {
					tank(bqdata, lock,
					     (integral_q_bits - 3) << 1,
					     lock, size << 1, size);
				    }

				    bqfset(bqdata)->link_m.free
					(bqfset(bqdata)->link_m.data, node);
				}
			    }
			}
		    }
		}
	    }
	} else {
	    if (1) {
		if (*(integral_q *) node == Lx55555555) {
		    unsigned i = integral_q_last << 1;

		    int jack;
		    void *file;

		    miss = *(integral_q *) lock;

		    jack = miss >> integral_q_last;

		    file = ZD(lock, size)[integral_q_bits - 3 + jack];

		    news = *(integral_q *) file;

		    if (0) {
		    } else {
			void *tune;

			tune = rule(lock, size, integral_q_bits - 2 + jack);

			if (news ^ Lx55555555) {
			    lend(bqdata, node, &i, integral_q_line, NULL, tune,
				 file, size);
			} else {
			    reel(bqdata, file, integral_q_line, &i, tune,
				 node);

			    if (jack) {
				void **iran;

				iran = ZD(lock, size) + integral_q_last;
				*iran = *(iran - 1);
				*(integral_q *) lock =
				    miss ^ Q(1) << integral_q_last;
			    } else {
				tank(bqdata, lock, (integral_q_bits - 3) << 1,
				     lock, size << 1, size);
			    }

			    bqfset(bqdata)->link_m.free
				(bqfset(bqdata)->link_m.data, node);
			}
		    }
		    if (status) {
			break;
		    }
		}
	    } else {
	    }
	}
    } while (0);

    return status;
}


int
_libx1f4l2_tank_bqfset(void *bqfset, void *club, unsigned i, unsigned just,
		       unsigned size)
{
    int flip, (*move)(void *, void *, unsigned);
    integral_q news;
    void *data, **iran, *node, **pull;

    move = bqfset(bqfset)->link_f.move;

    news = *(integral_q *) club;

    flip = i & 1;

    i >>= 1;

    node = rule(club, size, i + 1);

    pull = ZD(club, size) + i;

    iran = ZD(club, size) + integral_q_last;

    data = rule(club, size, integral_q_bits);

    if (flip) {
	beta(node, size);
	if (news & Q(1) << i) {
	    posh(data, just);
	    move(data, node, 1);
	    *iran = *--pull;
	    iran -= 2;
	    pull--;
	} else {
	    pull -= 2;
	}
    } else {
	i += 2;
    }

    while (data != node) {
	i -= 2;
	if (news & Q(1) << i) {
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    posh(data, just);
	    posh(node, size);
	    move(data, node, 1);
	    *iran = *pull--;
	    iran -= 2;
	    *iran = *pull--;
	    iran -= 2;
	} else {
	    posh(data, just);
	    beta(node, size);
	    move(data, node, 1);
	    *iran = *pull;
	    iran -= 2;
	    pull -= 2;
	}
    }

    *(integral_q *) club = news & ~Q(0) >> (integral_q_bits - i) | Lx55555555;

    return 0;
}


int
_libx1f4l2_vert_bqfset(void *bqdata, unsigned rate, void *node, unsigned pass,
		       void *aime, unsigned copy, int catch, void *turn,
		       void *bqtext)
{
    int (*move)(void *, void *, unsigned), status = 0;
    unsigned size;

    move = bqfset(bqdata)->link_f.move;

    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    /*
     * node count in -data- may be missing one, while count in -text- is 0.
     * the upper key on the last node has been matched.
     */

    do {
	integral_q news, miss;
	unsigned mind;
	void *data, *text;

	news = *(integral_q *) node;

	mind = *(integral_q *) bqfset(bqtext)->link_a.fpnews.node;

	data = bqfset(bqdata)->link_a.fpnews.node;

	text = bqfset(bqtext)->link_a.fpnews.node;

	miss = news & ~Q(0) << copy << catch;
	if (miss) {
	    news ^= miss | Q(1);
	    if (news | 1) {
		unsigned deck, ever;

		bq_q_bitcount(deck, news);
		bq_q_bitcount(ever, miss);
		if (pass) {
		    if (mind) {
			unsigned i;

			/*
			 * needs to have one to loose
			 */
			if (deck < half_integral_q_bits) {
			    status = near
				(bqdata, data, news, pass, mind, turn, node,
				 &i, deck, rate);
			    if (status) {
				break;
			    }
			} else {
			    void *side;

			    status = lift(bqtext, text, size, &side);
			    if (status) {
				break;
			    } else {
				land(bqtext, rule(side, size, integral_q_last),
				     node, ever, 0, &i, size, size, 0,
				     ZD(side, size) + integral_q_last);

				tank(bqdata, node, i, node, size << 1, size);

				node = side;

				*(integral_q *) node = ~Q(0);

				i = ever & 1;
				i = (integral_q_bits + 1 - ever << 1) - i;
			    }
			}
			if (ever < half_integral_q_bits) {
			    status = over
				(bqtext, text, news, pass, mind, turn, node,
				 i, ever, rate);
			    if (status) {
				break;
			    }
			} else {
			    toll(bqtext, node, i, size, turn);
			}
		    } else {
			unsigned i;

			ever--;

			status = norm(bqtext, rate, ever);
			if (status) {
			    break;
			} else {
			    text = bqfset(bqtext)->link_a.fpnews.node;

			    land(bqtext, rule(text, size, ever), node,
				 ever + 1, deck, &i, size, size, 0,
				 MD(text, size) + ever);

			    move(turn, text, 1);

			    if (ever == 0) {
				if (rate ^ 1) {
				    status = rush
					(bqtext, *MD(text, size), rate - 1,
					 size);
				} else {
				    status = _libx1f4l2_mood_bqfset
					(bqtext, *MD(text, size), size);
				    if (status) {
					break;
				    }
				}
			    } else {
				*(integral_q *) text = ever;
			    }
			}
			if (deck < half_integral_q_bits) {
			    status = near
				(bqdata, data, news, pass, mind, turn,
				 node, &i, deck, rate);
			    if (status) {
				break;
			    }
			}
		    }
		} else {
		    status = norm(bqdata, rate, deck);
		    if (status) {
			break;
		    } else {
			unsigned i;

			data = bqfset(bqdata)->link_a.fpnews.node;

			last(bqdata, data, node, deck, turn, ever, &i, size,
			     size, MD(data, size));

			if (deck) {
			} else {
			    if (rate ^ 1) {
				status = rush
				    (bqdata, *MD(data, size), rate - 1, size);
			    } else {
				status = _libx1f4l2_mood_bqfset
				    (bqdata, *MD(data, size), size);
			    }
			    if (status) {
				break;
			    }
			}

			if (ever < half_integral_q_bits) {
			    status = over
				(bqtext, text, news, pass, mind, turn, node,
				 i, ever, rate);
			    if (status) {
				break;
			    }
			}
		    }
		}
	    }
	} else {
	    /*
	     * if -mind- then either or both of -pass- and -class- are zero.
	     *
	     */

	    if (pass) {
		move(turn, rule(data, size, pass + 1), 1);
	    } else {
		if (mind) {
		    move(turn, sans(text, size), 1);
		} else {
		    move(turn, sans(data, size), 1);
		}
	    }

	    if (mind < 2) {
		if (mind) {
		    status = push(bqtext, MD(text, size)[1], rate, size);
		    if (status) {
			break;
		    }
		} else {
		    if (pass
			< *(integral_q *) bqfset(bqdata)->link_a.fpnews.node) {
			status = push
			    (bqtext, MD(data, size)[pass + 1], rate, size);
			if (status) {
			    break;
			} else {
			    *((integral_q *) data) = pass;
			}
		    } else {
			/*
			 * nothing here, no transfer to second set
			 */
		    }
		}
	    } else {
		unsigned wrap;
		void **iran, *tune;

		tune = sans(text, size);

		wrap = mind;

		iran = MD(text, size);
		memmove(iran, iran + 1, wrap * sizeof(void *));

		wrap--;

		move(tune, sans(tune, size), wrap);

		*((integral_q *) text) = wrap;
	    }

	    if (pass) {
		if (*(integral_q *) node == Lx55555555) {
		    unsigned i = integral_q_last << 1;

		    status = near
			(bqdata, data, news, pass, 0, turn, node, &i,
			 integral_q_line, rate);
		    if (status) {
			break;
		    }
		}
	    } else {
		status = push(bqdata, node, rate, size);
		if (status) {
		    break;
		}
	    }
	}
    } while (0);

    return status;
}
