/*
 * bqfv.f.c
 * Copyright (C) 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <bqfx-config.h>

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include <bqfx-defs.h>
#include <bqfx-inter.h>
#include <bqfx-lines.h>
#include <bqfx-names.h>
#include <bqfx-types.h>

#define rule(___s, ___n, ___t) \
    ((char *) (___s) + (___n) * (___t))

#define sans(___s, ___n) \
    ((char *) (___s) + (___n))

#define bqfset(bqfset)			((struct bqfset_type *) (bqfset))

static int clip(void *, unsigned, unsigned, int, int, void *);
static int fast(struct bqfset_type *, struct bqfset_type *);
static int lose(void *, void *, unsigned, void *, int, unsigned, void *,
		void *);
static int miss(void *, void *, unsigned, unsigned, void *, int, int, void *);
static int norm(void *, unsigned);
static int shut(void *, unsigned, unsigned, int, int, void *, void *);

static int
clip(void *bqdata, unsigned mind, unsigned copy, int clear, int catch,
     void *bqtext)
{
    int status = 0;

    if (catch) {
	if (clear) {
	} else {
	    copy--;
	}
    }

    if (copy) {
	unsigned yank;

	yank = mind - copy;
	if (yank) {
	    do {
		unsigned size;
		void *data, *text;

		size = bqfset(bqdata)->link_a.fpnews.link_v.size;

		norm(bqtext, yank);

		data = bqfset(bqdata)->link_a.fpnews.node;
		text = bqfset(bqtext)->link_a.fpnews.node;

		status = bqfset(bqtext)->link_f.move
		    (sans(text, size), rule(data, size, copy + 1), yank);
		if (status) {
		    status = MOVE_ERROR;
		    if (1) {
			break;
		    }
		}

		norm(bqdata, copy);
	    } while (0);
	} else {
	}
    } else {
	struct fpnews_type fpnews;

	fpnews = bqfset(bqdata)->link_a.fpnews;
	bqfset(bqdata)->link_a.fpnews = bqfset(bqtext)->link_a.fpnews;
	bqfset(bqtext)->link_a.fpnews = fpnews;
    }

    return status;
}


static int
fast(struct bqfset_type *bqfset_data, struct bqfset_type *bqfset_fast)
{
    int status;
    unsigned size;
    void *fset;

    *bqfset_fast = *bqfset_data;

    size = bqfset_fast->link_a.fpnews.link_v.size;

#if __MICRO_FSETS__
    size *= 3;
#else
    size += SIZEOF_integral_q;
    size *= integral_q_bits_and_half;
#endif				/* __MICRO_FSETS__ */

    status = bqfset_fast->link_m.link(bqfset_fast->link_m.data, &fset, size);
    if (status) {
	status = LINK_ERROR;
    } else {
	bqfset_fast->link_a.fpnews.node = fset;

	bqfset_fast->link_a.fpnews.link_v.rate = 0;

	*((integral_q *) fset) = 0;
    }

    return status;
}


static int
lose(void *bqdata, void *lock, unsigned rate, void *aime, int clear,
     unsigned size, void *turn, void *bqtext)
{
    int status;
    struct bqfset_type bqfset;

    do {
	int catch;
	unsigned copy;
	void *node;

	bqfset.link_a.fpnews.node = NULL;

	status = fast(bqdata, &bqfset);
	if (status) {
	    status = LINK_ERROR;
	}

	node = ZD(lock, size)[integral_q_last];

	catch = bqfset(bqdata)->link_f.pick(aime, node, &copy);
	copy -= ~*(integral_q *) node >> copy & 1;

	rate--;
	if (rate) {
	    int chase;

	    chase = catch;
	    catch = catch ? 0 : 1;

	    status = _libx1f4l2_bleu_bqfset
		(bqdata, lock, rate, node, aime, copy, catch, turn, &bqfset);
	    if (status) {
		break;
	    }

	    if (*(integral_q *) bqfset.link_a.fpnews.node) {
		void *trip;

		status = _libx1f4l2_post_bqfset
		    (chase & clear ? bqdata : &bqfset, turn, &trip);
		if (status) {
		    break;
		} else {
		    bqfset(bqtext)->link_f.move(trip, turn, 1);
		}
	    }

	    if (chase) {
	    } else {
		status = lose
		    (bqdata, ZD(lock, size)[integral_q_last], rate, aime,
		     clear, size, turn, &bqfset);
	    }
	} else {
	    catch = catch ? clear : 1;

	    status = _libx1f4l2_noir_bqfset
		(bqdata, lock, node, aime, copy, catch, turn, &bqfset);
	}

	if (status) {
	} else {
	    status = _libx1f4l2_join_bqfset(&bqfset, bqtext);
	    if (status) {
	    } else {
		struct fpnews_type fpnews;

		fpnews = bqfset.link_a.fpnews;
		bqfset.link_a.fpnews = bqfset(bqtext)->link_a.fpnews;
		bqfset(bqtext)->link_a.fpnews = fpnews;
	    }
	}
    } while (0);

    if (bqfset.link_a.fpnews.node) {
	_libx1f4l2_flat_bqfset(&bqfset);
    }

    return status;
}


static int
miss(void *bqdata, void *node, unsigned rate, unsigned pass, void *aime,
     int clear, int class, void *bqtext)
{
    int status = 0;
    void *turn = NULL;

    /*
     * node count in -data- may be missing one, while count in -text- is 0.
     * the upper key on the last node has been matched.
     */

    do {
	int catch;
	unsigned copy;

	turn = malloc(bqfset(bqdata)->link_a.fpnews.link_v.size * 2);
	if (turn) {
	} else {
	    status = LINK_ERROR;
	    break;
	}

	if (rate) {
	    int chase;

	    if (class) {
		copy = integral_q_last;
		catch = 1;
		chase = 1;
	    } else {
		integral_q news;

		news = *(integral_q *) node;

		catch = bqfset(bqdata)->link_f.pick(aime, node, &copy);
		chase = catch;
		copy -= ~news >> copy & 1;
		catch = catch ? 0 : 1;
	    }

	    status = _libx1f4l2_vert_bqfset
		(bqdata, rate, node, pass, aime, copy, catch, turn, bqtext);
	    if (status) {
		break;
	    }

	    if (*(integral_q *) bqfset(bqtext)->link_a.fpnews.node) {
		void *trip;

		status = _libx1f4l2_post_bqfset
		    (chase & clear ? bqdata : bqtext, turn, &trip);
		if (status) {
		    break;
		} else {
		    bqfset(bqtext)->link_f.move(trip, turn, 1);
		    if (chase) {
			break;
		    }
		}
	    }

	    if (rate < bqfset(bqdata)->link_a.fpnews.link_v.rate) {
		unsigned size;
		void *data;

		data = bqfset(bqdata)->link_a.fpnews.node;

		pass = *(integral_q *) bqfset(bqdata)->link_a.fpnews.node;
		size = bqfset(bqdata)->link_a.fpnews.link_v.size;

		status = lose
		    (bqdata, MD(data, size)[pass], rate, aime, clear, size,
		     turn, bqtext);
	    } else {
		struct bqfset_type bqfset;

		status = fast(bqdata, &bqfset);
		if (status) {
		    status = LINK_ERROR;
		} else {
		    status = _libx1f4l2_tear_bqfset
			(bqdata, aime, clear, &bqfset);
		    if (status) {
		    } else {
			status = _libx1f4l2_join_bqfset(&bqfset, bqtext);
			if (status) {
			} else {
			    struct fpnews_type fpnews;

			    fpnews = bqfset.link_a.fpnews;
			    bqfset.link_a.fpnews =
				bqfset(bqtext)->link_a.fpnews;
			    bqfset(bqtext)->link_a.fpnews = fpnews;
			}
		    }

		    _libx1f4l2_flat_bqfset(&bqfset);
		}
	    }
	} else {
	    if (class) {
		copy = integral_q_last;
		catch = 1;
	    } else {
		integral_q news;

		news = *(integral_q *) node;

		catch = bqfset(bqdata)->link_f.pick(aime, node, &copy);
		copy -= ~news >> copy & 1;
		catch = catch ? clear : 1;
	    }

	    status = _libx1f4l2_gris_bqfset
		(bqdata, node, pass, aime, clear, class, copy, catch, turn,
		 bqtext);
	}
    } while (0);

    if (turn) {
	free(turn);
    }

    return status;
}


static int
norm(void *bqfset, unsigned fold)
{
    int status = 0;
#if __MICRO_FSETS__
    unsigned fail, sans, size;
    void *node;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    size = bqfset(bqfset)->link_a.fpnews.link_v.size;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    node = bqfset(bqfset)->link_a.fpnews.node;
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    fail = fold < 3 ? 3 : fold;
    sans = integral_q_bits_and_half;
    for (; fail < sans; sans >>= 1);
    sans <<= 1;
    if (sans == integral_q_bits_and_half) {
	sans *= (size + SIZEOF_integral_q);
    } else {
	sans *= size;
    }
#endif				/* __MICRO_FSETS__ */

#if __MICRO_FSETS__
    /*
     * resizing no matter whether is required.  and to minimum 6.
     */
    status = bqfset(bqfset)->link_m.mode
	(bqfset(bqfset)->link_m.data, &node, sans);
    if (status) {
	if (bqfset(bqfset)->link_a.fpnews.link_v.rate) {
	    *((integral_q *) node) = fold;

	    status = 0;
	} else {
	    status = MODE_ERROR;
	}
    } else {
#endif				/* __MICRO_FSETS__ */
	*((integral_q *) node) = fold;

#if __MICRO_FSETS__
	bqfset(bqfset)->link_a.fpnews.node = node;
    }
#endif				/* __MICRO_FSETS__ */

    return status;
}


static int
shut(void *bqdata, unsigned mind, unsigned copy, int clear, int catch,
     void *bqtext, void *aime)
{
    int status;
    unsigned rate, size, wrap;
    void *data, *node, *text;

    rate = bqfset(bqdata)->link_a.fpnews.link_v.rate;
    size = bqfset(bqdata)->link_a.fpnews.link_v.size;

    data = bqfset(bqdata)->link_a.fpnews.node;
    text = bqfset(bqtext)->link_a.fpnews.node;

    node = MD(data, size)[copy - catch];

    wrap = mind - copy;

    do {
	if (wrap) {
	    int (*move)(void *, void *, unsigned);

	    move = bqfset(bqdata)->link_f.move;

	    if (catch < copy) {
		void *trip, *turn;

#if __MICRO_FSETS__
		status = bqfset(bqtext)->link_m.mode
		    (bqfset(bqtext)->link_m.data, &text,
		     integral_q_bits_and_half * (size + SIZEOF_integral_q));
		if (status) {
		    status = MODE_ERROR;
		    if (1) {
			break;
		    }
		} else {
		    bqfset(bqtext)->link_a.fpnews.node = text;
		}
#endif				/* __MICRO_FSETS__ */

		turn = rule(data, size, copy);

		move(sans(text, size), sans(turn, size), wrap);
		memcpy(MD(text, size), MD(data, size) + copy,
		       (wrap + 1) * sizeof(void *));

		*((integral_q *) text) = wrap;
		*((integral_q *) data) = copy - catch;

		bqfset(bqtext)->link_a.fpnews.link_v.rate = rate;

		if (catch) {
		    status = _libx1f4l2_post_bqfset
			(clear ? bqdata : bqtext, turn, &trip);
		    if (status) {
		    } else {
			move(trip, turn, 1);
		    }

		    break;
		} else {
		}
	    } else {
		struct fpnews_type fpnews;

		fpnews = bqfset(bqdata)->link_a.fpnews;
		bqfset(bqdata)->link_a.fpnews = bqfset(bqtext)->link_a.fpnews;
		bqfset(bqtext)->link_a.fpnews = fpnews;

		copy = 0;
	    }
	} else {
	    if (catch) {
		copy--;
		if (rate ^ 1) {
		} else {
		    *((integral_q *) data) = copy;
		}
	    } else {
	    }
	}

	rate--;

	status = miss(bqdata, node, rate, copy, aime, clear, catch, bqtext);
    } while (0);

    return status;
}


int
x1f4_tear_bqfset(void *bqdata, void *aime, int clear, void *bqtext)
{
    int status;
    unsigned mind;

    mind = *(integral_q *) bqfset(bqtext)->link_a.fpnews.node;
    if (mind) {
	status = SIZE_ERROR;
    } else {
	int catch;
	unsigned copy, rate;

	catch = bqfset(bqdata)->link_f.look
	    (aime, bqfset(bqdata)->link_a.fpnews.node, &copy);

	mind = *(integral_q *) bqfset(bqdata)->link_a.fpnews.node;

	rate = bqfset(bqdata)->link_a.fpnews.link_v.rate;
	if (rate) {
	    clear = clear ? 1 : 0;
	    catch = catch ? 1 : 0;
	    status = shut(bqdata, mind, copy, clear, catch, bqtext, aime);
	} else {
	    status = clip(bqdata, mind, copy, clear, catch, bqtext);
	}
    }

    return status;
}
