/*
 * e4-o.5.c
 * Copyright (C) 2007-2015, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>

#include <e4-types.h>

extern const char *_x1f4_e4_type_names[];
extern const unsigned _x1f4_e4_type_sizes[];

static void float_name(void *, unsigned, const char *, unsigned);
static void print_mark(void *, const struct e4_odbx_type *);
static void print_type(void *, int, unsigned, const void *,
		       int (*)(const void *, int, const char **, unsigned *));

static const char space[] = "                ";

static void
float_name(void *file, unsigned span, const char *name, unsigned size)
{
    unsigned left;

    if (size < span) {
	span -= size;
    } else {
	span = 0;
    }

    left = span >> 1;
    span -= left;

    fwrite(space, left, 1, file);

    fwrite(name, size, 1, file);

    fwrite(space, span, 1, file);
}


static void
print_mark(void *file, const struct e4_odbx_type *odbx_data)
{
    do {
	if (odbx_data->bits & E2ND_LINK) {
	    const struct e4_2nde_type *e2nd_data;

	    e2nd_data = odbx_data->extension2;
	    if (e2nd_data->bits & POST_MARK) {
		const struct e4_mark_type *mark_data;

		mark_data = e2nd_data->post;
		float_name(file, 8, mark_data->part, mark_data->size);

		break;
	    }
	}

	fwrite(space, 8, 1, file);
    } while (0);
}


static void
print_type(void *file, int type, unsigned span, const void *context,
	   int (*fix)(const void *, int, const char **, unsigned *))
{
    if (type < SUIT) {
	if (type < EVER) {
	    float_name
		(file, span, _x1f4_e4_type_names[type],
		 _x1f4_e4_type_sizes[type]);
	} else {
	    float_name(file, span, "", 0);
	}
    } else {
	if (fix) {
	    const char *name;
	    unsigned size;

	    if (fix(context, type, &name, &size)) {
		if (BASE_LOCK == type) {
		    float_name(file, span, "<function>", 10);
		} else {
		    float_name(file, span, "<user>", 6);
		}
	    } else {
		float_name(file, span, name, size);
	    }
	} else {
	    float_name(file, span, "<user>", 6);
	}
    }
}


void
x1f4_print_tboperators(FILE *file, const struct e4_odbx_type *const *list,
		       int head, const void *context,
		       int (*fix)(const void *, int, const char **,
				  unsigned *))
{
    const struct e4_odbx_type *odbx_data;

    if (head) {
	fputs("      Type        Left Operand           Right Operand      "
	      "     Precedence\n", file);
    }

    odbx_data = *list;
    while (odbx_data) {
	const int *args;

	if (odbx_data->bits & E2ND_LINK) {
	    if (((struct e4_2nde_type *) odbx_data->extension2)->bits
		& RAIL_ONLY) {
		goto shift;
	    }
	}

	print_type(file, odbx_data->type, 16, context, fix);
	args = odbx_data->args;
	print_type(file, *args, 16, context, fix);
	float_name(file, 8, odbx_data->name, odbx_data->size);
	args++;
	print_type(file, *args, 16, context, fix);
	print_mark(file, odbx_data);
	fprintf(file, "    %#4o\n", odbx_data->pass);

      shift:
	list++;
	odbx_data = *list;
    }
}
