/*
 * e4.7.5.c
 * Copyright (C) 2006-2014, 2017, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <e4-parse.h>
#include <e4-types.h>
#include <e4fine-e4.h>

#define parser(text) \
    ((struct parser_type *) (text))

int
_x1f4_e4_pick_odaa(const struct e4_odbx_type **odbx, unsigned toxic,
		   const char *f, unsigned length, void *parser)
{
    const struct e4_odbx_type *const *list, *odbx_data;

    list = ((struct parser_type *) parser)->context_data->odb2_data;

    /*
     * will not take an operator of matching length with the closing
     * ternary.
     */

    odbx_data = *list;
    while (odbx_data) {
	unsigned shift;

	shift = odbx_data->size;
	if (toxic < shift) {
	    if (length < shift) {
	    } else {
		if (memcmp((void *) f, odbx_data->name, shift)) {
		} else {
		    *odbx = odbx_data;
		    toxic = shift - 1;

		    if (odbx_data->bits & E2ND_LINK) {
			const struct e4_2nde_type *e2nd_data;
			unsigned bits;

			e2nd_data = odbx_data->extension2;
			bits = e2nd_data->bits;
			if (bits & (RAIL_LIST | RAIL_MARK)) {
			    if (bits & RAIL_ONLY) {
			    } else {
				toxic = shift;
			    }
			}
		    }
		}
	    }
	}

	list++;
	odbx_data = *list;
    }

    return 0;
}


int
_x1f4_e4_pick_odae(const struct e4_odbx_type **odbx, int *lock_type,
		   int major, int minor, const char *name, unsigned length,
		   void *parser)
{
    const struct e4_odbx_type *const *list, *odbx_data;

    list = ((struct parser_type *) parser)->context_data->odb2_data;

    odbx_data = *list;
    while (odbx_data) {
	unsigned size;

	size = odbx_data->size;

	if (odbx_data->bits & E2ND_LINK) {
	    if (((struct e4_2nde_type *) odbx_data->extension2)->bits
		& RAIL_ONLY) {
		size = 0;
	    }
	}

	if (size == length) {
	    const int *args;

	    args = odbx_data->args;
	    if (args[0] == major) {
		if (args[1] == minor) {
		    if (memcmp(odbx_data->name, name, length)) {
		    } else {
			*lock_type = odbx_data->type;
			break;
		    }
		}
	    }
	    if ((odbx_data->bits & (CALL_LOCK | LEFT_XSET))
		== (CALL_LOCK | LEFT_XSET) && BASE_LOCK < major) {
		if (BASE_LOCK < minor || args[1] == minor) {
		    if (memcmp(odbx_data->name, name, length)) {
		    } else {
			if (BASE_LOCK < minor) {
			    if (major == minor) {
				*lock_type = major;
				break;
			    } else {
				if (parser(parser)->e4fine) {
				    if (x1f4_void_e4fine
					(parser(parser)->e4fine, major,
					 minor)) {
					*lock_type = major;
					break;
				    }
				}
			    }
			} else {
			    if (args[1] == minor) {
				*lock_type = major;
				break;
			    }
			}
		    }
		} else {
		}
	    }
	}

	list++;
	odbx_data = *list;
    }

    *odbx = odbx_data;

    return 0;
}


int
_x1f4_e4_pick_odta(const struct e4_odbx_type **odbu,
		   const struct e4_odbx_type **odbx, unsigned toxic,
		   const char *f, unsigned length, const char *name,
		   unsigned size, void *parser)
{
    const struct e4_odbx_type *const *list, *odbx_case;
    unsigned shift;

    list = ((struct parser_type *) parser)->context_data->odb2_data;

    odbx_case = *list;
    while (odbx_case) {
	if (odbx_case->bits & E2ND_LINK) {
	    const struct e4_2nde_type *e2nd_data;
	    unsigned bits;

	    e2nd_data = odbx_case->extension2;
	    bits = e2nd_data->bits;
	    if (bits & (RAIL_LIST | RAIL_MARK) && odbx_case->size == size) {
		if (memcmp(odbx_case->name, name, size)) {
		} else {
		    if (bits & RAIL_MARK) {
			const struct e4_odbx_type *odbx_data;

			odbx_data = e2nd_data->rail;
			shift = odbx_data->size;
			if (toxic < shift) {
			    if (length < shift) {
			    } else {
				if (memcmp(f, odbx_data->name, shift)) {
				} else {
				    *odbu = odbx_case;
				    *odbx = odbx_data;
				}
			    }
			}
		    } else {
			const struct e4_odbx_type *odbx_data;
			const struct e4_rail_type *rail_data;
			unsigned i;

			rail_data = e2nd_data->rail;
			i = rail_data->miss;
			odbx_data = rail_data->odbx_data;
			for (; i; i--) {
			    shift = odbx_data->size;
			    if (toxic < shift) {
				if (length < shift) {
				} else {
				    if (memcmp(f, odbx_data->name, shift)) {
				    } else {
					*odbu = odbx_case;
					*odbx = odbx_data;
				    }
				}
			    }

			    odbx_data++;
			}
		    }
		}
	    }
	}

	list++;
	odbx_case = *list;
    }

    return 0;
}


int
_x1f4_e4_pick_odte(const struct e4_odbx_type **odbx, int *lock_type,
		   int major, int minor, const char *name, unsigned length,
		   void *parser)
{
    const struct e4_odbx_type *const *list, *odbx_data;

    list = ((struct parser_type *) parser)->context_data->odb2_data;

    odbx_data = *list;
    while (odbx_data) {
	if (odbx_data->bits & E2ND_LINK) {
	    const struct e4_2nde_type *e2nd_data;

	    e2nd_data = odbx_data->extension2;
	    if (e2nd_data->bits & (RAIL_LIST | RAIL_MARK)) {
		if (odbx_data->size == length) {
		    const int *args;

		    args = odbx_data->args;
		    if (args[0] == major) {
			if (args[1] == minor) {
			    if (memcmp(odbx_data->name, name, length)) {
			    } else {
				*lock_type = odbx_data->type;
				break;
			    }
			}
		    }
		}
	    }
	}

	list++;
	odbx_data = *list;
    }

    *odbx = odbx_data;

    return 0;
}
