/*
 * lxcase-a.a.c
 * Copyright (C) 2008-2014, 2016, 2019, 2021, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <signal.h>
#include <unistd.h>

#include <sys/stat.h>

#include <e4.h>
#include <lxcase-defs.h>
#include <variable.h>

typedef struct constant_type {
    struct x1f4_variable_type variable;
    const void *data;
} constant_type;

static X1f4_E4_C_MODE names[] = {
/* *INDENT-OFF* */
    SIGABRT,
    SIGALRM,
    SIGCHLD,
    SIGCONT,
    SIGFPE,
    SIGHUP,
    SIGILL,
    SIGINT,
    SIGKILL,
    SIGPIPE,
    SIGQUIT,
    SIGSEGV,
    SIGSTOP,
    SIGTERM,
    SIGTSTP,
    SIGTTIN,
    SIGTTOU,
    SIGUSR1,
    SIGUSR2,
    F_OK,
    R_OK,
    W_OK,
    X_OK,
    ST_ATIME,
    ST_CTIME,
    ST_GID,
    ST_MODE,
    ST_MTIME,
    ST_NLINK,
    ST_SIZE,
    ST_UID,
    SU_ATIME,
    SU_CTIME,
    SU_MTIME,
    S_IFMT,
    S_IFDIR,
    S_IFCHR,
    S_IFBLK,
    S_IFREG,
    S_IFLNK,
    S_IFIFO,
    S_IFSOCK,
/* *INDENT-ON* */
};
static const struct constant_type constants[] = {
/* *INDENT-OFF* */
    {	{	"F_OK",			X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	4			},
	names + 19							},
    {	{	"R_OK",			X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	4			},
	names + 20							},
    {	{	"SIGABRT",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 0							},
    {	{	"SIGALRM",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 1							},
    {	{	"SIGCHLD",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 2							},
    {	{	"SIGCONT",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 3							},
    {	{	"SIGFPE",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 4							},
    {	{	"SIGHUP",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 5							},
    {	{	"SIGILL",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 6							},
    {	{	"SIGINT",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 7							},
    {	{	"SIGKILL",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 8							},
    {	{	"SIGPIPE",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 9							},
    {	{	"SIGQUIT",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 10							},
    {	{	"SIGSEGV",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 11							},
    {	{	"SIGSTOP",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 12							},
    {	{	"SIGTERM",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 13							},
    {	{	"SIGTSTP",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 14							},
    {	{	"SIGTTIN",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 15							},
    {	{	"SIGTTOU",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 16							},
    {	{	"SIGUSR1",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 17							},
    {	{	"SIGUSR2",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 18							},
    {	{	"W_OK",			X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	4			},
	names + 21							},
    {	{	"X_OK",			X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	4			},
	names + 22							},
    {	{	"ST_ATIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 23							},
    {	{	"ST_CTIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 24							},
    {	{	"ST_GID",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 25							},
    {	{	"ST_MODE",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 26							},
    {	{	"ST_MTIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 27							},
    {	{	"ST_NLINK",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 28							},
    {	{	"ST_SIZE",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 29							},
    {	{	"ST_UID",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 30							},
    {	{	"SU_ATIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 31							},
    {	{	"SU_CTIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 32							},
    {	{	"SU_MTIME",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 33							},
    {	{	"S_IFMT",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	6			},
	names + 34							},
    {	{	"S_IFDIR",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 35							},
    {	{	"S_IFCHR",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 36							},
    {	{	"S_IFBLK",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 37							},
    {	{	"S_IFREG",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 38							},
    {	{	"S_IFLNK",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 39							},
    {	{	"S_IFIFO",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	7			},
	names + 40							},
    {	{	"S_IFSOCK",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	8			},
	names + 41							},
/* *INDENT-ON* */
};

int
x1f4_vset_lxcase(void *lxcase, void **context)
{
    int status;
    const struct constant_type *constant_data;
    unsigned i;

    constant_data = constants;

    i = sizeof(constants) / sizeof(struct constant_type);
    for (; i; i--) {
	status = x1f4_push_variable
	    (context, constant_data->variable.name,
	     constant_data->variable.length, constant_data->variable.type,
	     constant_data->variable.flags, (void *) constant_data->data);
	if (status) {
	    break;
	} else {
	    constant_data++;
	}
    }

    return status;
}
