/*
 * lxcast-e.t.c
 * Copyright (C) 2010-2011, 2016, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include <dt.h>
#include <ix.h>
#include <lxcast-types.h>

#define ORDAIN_BITS \
    ((1 << X1f4_E4_BILL) | (1 << X1f4_E4_MODE) | (1 << X1f4_E4_TEXT))

#define NEAR_BITS \
    X1f4_E4_BACK_LINK | X1f4_E4_CALL_XSET | X1f4_E4_LEFT_XSET		      \
    | X1f4_E4_E2ND_LINK

#define lxcast(lxcast) \
    ((struct lxcast_type *) (lxcast))

extern const struct x1f4_linetext_type *_libx1f4i0_lxcast_fade_i[];

static int link_over(struct lxcast_type *, unsigned *,
		     const struct x1f4_linetext_type ***,
		     const struct x1f4_2ndscope_type *);

static int
link_over(struct lxcast_type *lxcast_data, unsigned *keep,
	  const struct x1f4_linetext_type ***turn,
	  const struct x1f4_2ndscope_type *scope2nd_data)
{
    int excess;
    unsigned over;
    void *line, **pick;

    line = (void *) *turn;
    over = *keep;
    if (over) {
	excess = lxcast_data->link_w.mode
	    (lxcast_data->link_w.data, &line, sizeof(void *) * (over + 2));
	if (excess) {
	} else {
	    (*keep)++;
	    pick = line;
	    pick += over;
	}
    } else {
	excess = lxcast_data->link_w.link
	    (lxcast_data->link_w.data, &line, sizeof(void *) << 1);
	if (excess) {
	} else {
	    *keep = 1;
	    pick = line;
	}
    }
    if (excess) {
    } else {
	*turn = line;
	*pick = (void *) scope2nd_data->line;
	pick[1] = (void *) _libx1f4i0_lxcast_fade_i[0];
    }

    return 0;
}


int
x1f4_wrap_lxcast(void *lxcast, struct x1f4_indexset_type *indexset_data)
{
#if 0
    const char *name;
    const struct x1f4_datatype_type *datatype_data;
#endif				/* 0 */
    const struct x1f4_operator_type *const *operator2s, *operator_data;
    unsigned hold[5][2];

#if 0
    datatype_data = indexset_data->datatype_set.datatype_data;

    name = datatype_data->name;
    while (name) {
	if (datatype_data->size == 4) {
	    int a, b, c, d;

	    a = name[0];
	    b = name[1];
	    c = name[2];
	    d = name[3];

	    if (a == 'd' && b == 'a' && c == 't' && d == 'a') {
		lxcast(lxcast)->link_l.data = datatype_data->context;
		break;
	    }
	}

	datatype_data++;

	name = datatype_data->name;
    }
#endif				/* 0 */

    operator2s = indexset_data->operator_set.operator2s;

    memset(hold, 0, sizeof hold);

    operator_data = *operator2s;
    while (operator_data) {
	const int *args;
	int type;

	args = operator_data->args;
	type = args[1];
	if (operator_data->length == 1 && *args ^ X1f4_E4_CASE
	    && operator_data->flags & NEAR_BITS == X1f4_E4_E2ND_LINK) {
	    int near;
	    unsigned post = 0;

	    if ((type < 4 && 1 << type & ORDAIN_BITS || type == X1f4_E4_CASE)
		&& *operator_data->name == '[') {
		post = X1f4_E4_POST_MARK;
		near = 1;
	    } else {
		if ((type == X1f4_E4_TEXT /* || type == X1f4_E4_CASE */)
		    && operator_data->flags & X1f4_E4_POST_COPY
		    && *operator_data->name == '`') {
		    near = 4;
		} else {
		    near = 0;
		}
	    }
	    if (near) {
		const struct x1f4_2ndscope_type *scope2nd_data;

		scope2nd_data = operator_data->extension2;
		if ((scope2nd_data->bits & (post | X1f4_E4_RAIL_ONLY)) == post
		    && scope2nd_data->bits & X1f4_E4_CALL_LINK) {
		    if (near == 1) {
			const struct x1f4_postmark_type *postmark_data;

			postmark_data = scope2nd_data->post;
			if (postmark_data->size == 1
			    && *postmark_data->mark == ']') {
			} else {
			    near = 0;
			}
		    }
		    if (near) {
			struct cxnear_type *cxnear_data;
			unsigned *keep;

			cxnear_data = lxcast(lxcast)->line_f.near;

			if (near == 1) {
			    switch (type) {
			    case X1f4_E4_BILL:
				near = 0;
				break;
			    case X1f4_E4_CASE:
				near = 3;
				break;
			    case X1f4_E4_MODE:
				near = 1;
				break;
			    case X1f4_E4_TEXT:
				near = 2;
				break;
			    }
			}

			cxnear_data += near;

			keep = hold[near];

			link_over
			    (lxcast, keep, &cxnear_data->pick, scope2nd_data);

			if (scope2nd_data->bits & X1f4_E4_RAIL_LIST) {
			    const struct x1f4_raillist_type *raillist_data;
			    unsigned miss;

			    raillist_data = scope2nd_data->rail;
			    operator_data = raillist_data->operator_data;
			    miss = raillist_data->miss;
			    for (; miss; miss--, operator_data++) {
				if (operator_data->args[1] == X1f4_E4_SLIP
				    && operator_data->flags & NEAR_BITS
				    == X1f4_E4_E2ND_LINK
				    && operator_data->type == X1f4_E4_CASE) {
				    scope2nd_data = operator_data->extension2;
				    link_over(lxcast, keep + 1,
					      &cxnear_data->rail,
					      scope2nd_data);
				}
			    }
			} else {
			    if (scope2nd_data->bits & X1f4_E4_RAIL_MARK) {
				operator_data = scope2nd_data->rail;
				if (operator_data->args[1] == X1f4_E4_SLIP
				    && operator_data->flags & NEAR_BITS
				    == X1f4_E4_E2ND_LINK
				    && operator_data->type == X1f4_E4_CASE) {
				    scope2nd_data = operator_data->extension2;
				    link_over(lxcast, keep + 1,
					      &cxnear_data->rail,
					      scope2nd_data);
				}
			    }
			}
		    }
		}
	    }
	}

	operator2s++;

	operator_data = *operator2s;
    }

    return 0;
}
