/*
 * lxcast-i.1.c
 * Copyright (C) 2008-2015, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <lxcall.h>
#include <lxcast-inter.h>
#include <lxcast-types.h>

#define LINE_D__BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK

#define SIDE_CE_BITS \
    X1f4_E4_E2ND_LINK

#define I_BILL(b)			(*((X1f4_E4_C_BILL *) (b)))
#define I_CASE(u)			(*((X1f4_E4_C_USER *) (u)))
#define I_MODE(i)			(*((X1f4_E4_C_MODE *) (i)))
#define I_REAL(r)			(*((X1f4_E4_C_REAL *) (r)))

#define l_BILL(e, output) \
    {									      \
	X1f4_E4_C_BILL *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_REAL(e, output) \
    {									      \
	X1f4_E4_C_REAL *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_USER(e, output) \
    {									      \
	X1f4_E4_C_USER *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define MAKE_SINGLE(a, b)		a

#define lxcast(lxcast) \
    ((struct lxcast_type *) (lxcast))

static int a61xx_c__(void *, void *, void **);
static int a6exx_c__(void *, void *, void **);
static int a6fxx_c__(void *, void *, void **);
static int a73xx_c__(void *, void *, void **);

static const int c_____c__[] = {
/* *INDENT-OFF* */
    X1f4_E4_CASE
/* *INDENT-ON* */
};
const struct x1f4_function_type _libx1f4i0_lxcast_line_u[] = {
/* *INDENT-OFF* */
    {	"over",			a61xx_c__,
	X1f4_E4_CASE,		c_____c__,	1,
	LINE_D__BITS,				4		},
    {	"unit",			a6exx_c__,
	X1f4_E4_MODE,		c_____c__,	1,
	LINE_D__BITS,				4		},
    {	"over",			a6fxx_c__,
	X1f4_E4_CASE,		c_____c__,	1,
	LINE_D__BITS,				4		},
    {	"over",			a73xx_c__,
	X1f4_E4_CASE,		c_____c__,	1,
	LINE_D__BITS,				4		}
/* *INDENT-ON* */
};
const struct x1f4_operator_type _libx1f4i0_lxcast_side_u[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("+", " "),	NULL,		0210,
	X1f4_E4_CASE,		c_____c__,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("!", " "),	NULL,		0210,
	X1f4_E4_MODE,		c_____c__,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("~", " "),	NULL,		0210,
	X1f4_E4_CASE,		c_____c__,
	SIDE_CE_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("-", " "),	NULL,		0210,
	X1f4_E4_CASE,		c_____c__,
	SIDE_CE_BITS,		1,
	NULL,			NULL				}
/* *INDENT-ON* */
};
const unsigned _libx1f4i0_lxcast_size_u =
    sizeof(_libx1f4i0_lxcast_side_u) / sizeof(struct x1f4_operator_type);

static int
a61xx_c__(void *context, void *output, void **input)
{
    int status;
    struct dxcast_type *object_data;

    object_data = I_CASE(input[0]);

    switch (object_data->type) {
    case X1f4_E4_BILL:
    case X1f4_E4_MODE:
    case X1f4_E4_REAL:
	status = 0;
	l_USER(object_data, output);
	break;
    default:
	status = _libx1f4i0_lxcast_stat_over(context, object_data->type);
    }

    return status;
}


static int
a6exx_c__(void *context, void *output, void **input)
{
    int unit, status = 0;
    struct dxcast_type *object_data;

    object_data = I_CASE(input[0]);

    switch (object_data->type) {
    case X1f4_E4_BILL:
	unit = !I_BILL(&object_data->data);
	break;
    case X1f4_E4_MODE:
	unit = !I_MODE(&object_data->data);
	break;
    case X1f4_E4_REAL:
	unit = !I_REAL(&object_data->data);
	break;
    default:
	unit = 0;
	status = _libx1f4i0_lxcast_stat_over(context, object_data->type);
    }

    l_MODE(unit, output);

    return status;
}


static int
a6fxx_c__(void *context, void *output, void **input)
{
    int status = 0, type;
    struct dxcast_type *object_data;
    struct fxdata_type fxdata;

    object_data = I_CASE(input[0]);

    type = object_data->type;

    switch (type) {
    case X1f4_E4_BILL:
	l_BILL(~I_BILL(&object_data->data), &fxdata);
	break;
    case X1f4_E4_MODE:
	l_MODE(~I_MODE(&object_data->data), &fxdata);
	break;
    default:
	status = 1;
    }

    if (status) {
	status = _libx1f4i0_lxcast_stat_over(context, object_data->type);
    } else {
	status = x1f4_cast_rate
	    (&lxcast(context)->link_v, &lxcast(context)->link_e, type, &fxdata,
	     output);
    }

    return status;
}


static int
a73xx_c__(void *context, void *output, void **input)
{
    int status = 0, type;
    struct dxcast_type *object_data;
    struct fxdata_type fxdata;

    object_data = I_CASE(input[0]);

    type = object_data->type;

    switch (type) {
    case X1f4_E4_BILL:
	l_BILL(-I_BILL(&object_data->data), &fxdata);
	break;
    case X1f4_E4_MODE:
	l_MODE(-I_MODE(&object_data->data), &fxdata);
	break;
    case X1f4_E4_REAL:
	l_REAL(-I_REAL(&object_data->data), &fxdata);
	break;
    default:
	status = 1;
    }

    if (status) {
	status = _libx1f4i0_lxcast_stat_over(context, object_data->type);
    } else {
	status = x1f4_cast_rate
	    (&lxcast(context)->link_v, &lxcast(context)->link_e, type, &fxdata,
	     output);
    }

    return status;
}
