/*
 * lxdash-f.p.c
 * Copyright (C) 2008-2015, 2017, 2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <copy.h>
#include <d_fset.h>
#include <e4.h>
#include <fd.h>
#include <lxcast.h>
#include <lxdash-close.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>
#include <nt.h>

int
_libx1f4i0_lxdash_p_forward(void *context, void *output, void **input)
{
    int status, type;
    struct dxdash_type *dxdash_data;
    struct lxdash_type *lxdash_data;
    struct lxtext_type *lxtext_data, *lxtext_link;
    unsigned miss;
    void *trap;

    lxdash_data = context;

    trap = input[2];

    type = ((struct x1f4_function_type *)
	    ((void **) input[-1])[X1f4_E4_RENDITION])->args[2];
    if (type == X1f4_E4_CASE) {
	struct x1f4_dxcast_type *dxcast_data;

	dxcast_data = I_USER(trap);
	trap = &dxcast_data->data;
	type = dxcast_data->type;
    }

    lxtext_link = lxdash_data->link_f.text;

    lxtext_data = lxtext_link;

    close_frame(miss, lxtext_data);

    dxdash_data = I_LINE(input[0]);

    do {
	int (*copy)(void *, void *, void **, const void *,
		    const struct x1f4_nodelink_type *);
	void *data, *side;

	if (miss) {
	    copy = lxtext_data->lxtype.copy;
	    if (copy) {
		status = copy
		    (lxtext_data->lxtype.context, NULL, &side, I_USER(trap),
		     lock_excase(dxdash_data));
		if (status) {
		    break;
		}
	    }
	} else {
	    copy = NULL;
	}

	if (0) {
	} else {
	    struct effect_type effect;

	    effect.mode = I_MODE(input[1]);
	    status = x1f4_post_lxfset(dxdash_data->dash, &effect, &data);
	    if (status) {
		status = _libx1f4i0_lxdash_stat_text
		    (lxdash_data, status, effect.mode);

		if (copy) {
		    lxtext_data->lxtype.free
			(lxtext_data->lxtype.context, side,
			 NULL, lock_excase(dxdash_data));
		}
	    } else {
		struct effect_type *effect_data;

		effect_data = data;

		effect_data->mode = effect.mode;
		effect_data->type = type;
		effect_data->link = miss ? lxtext_data - lxtext_link : ~0;

		dxdash_data->size++;

		if (copy) {
		    l_USER(side, &effect_data->fxdata);
		} else {
		    copy_miss(&effect_data->fxdata, type, trap);
		}
	    }
	}
    } while (0);

    return status;
}
