/*
 * lxdash-k.i.c
 * Copyright (C) 2008-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxdash-config.h>

#include <copy.h>
#include <d_fset.h>
#include <e4.h>
#include <fd.h>
#include <lxdash-drive.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

int
_libx1f4i0_lxdash_i_keeping(void *context, void *output, void **input)
{
    int status;
    struct cxbend_type *cxbend_data;
    struct dxdash_type *dxdash_data;
    struct lxdash_type *lxdash_data;
    void *dash, *data;

    cxbend_data = context;

    lxdash_data = cxbend_data->line;

    dxdash_data = I_LINE(input[1]);

    dash = dxdash_data->dash;

    status = cxbend_data->pick(dash, &data);
    if (status) {
	status = _libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
    } else {
	struct effect_type *effect_data;
	unsigned call, type;

	effect_data = data;

	type = effect_data->type;
	call = *((struct x1f4_function_type *)
		 ((void **) input[-1])[X1f4_E4_RENDITION])->args;
	if (type != call) {
	    status = _libx1f4i0_lxdash_stat_ends
		(lxdash_data, cxbend_data->bend, type, call);
	} else {
	    unsigned link;

	    link = effect_data->link;

	    do {
		if (~link) {
		    struct lxtext_type *lxtext_data;

		    lxtext_data = lxdash_data->link_f.text;
		    lxtext_data += link;
		    if (lxtext_data->lxtype.push) {
			status = lxtext_data->lxtype.push
			    (lxtext_data->lxtype.context, *input,
			     I_USER(&effect_data->fxdata),
			     lock_excase(dxdash_data));
			if (status) {
			    break;
			}
		    } else {
			copy_miss(*input, type, &effect_data->fxdata);
		    }

		    effect_data->link = ~0;
		} else {
		    copy_miss(*input, type, &effect_data->fxdata);
		}

		status = cxbend_data->slip(dash);
		if (status) {
		    status =
			_libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
		} else {
		    dxdash_data->size--;

		    status = _libx1f4i0_lxrail_cast_data
			(lxdash_data, output, type, *input);
		}
	    } while (0);
	}
    }

    return status;
}
