/*
 * lxdata-a.j.c
 * Copyright (C) 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <lxdata-types.h>
#include <progress.h>

#define DATA_PROGRESS_POSITION \
    X1f4_PROGRESS_EQCLAUSE | X1f4_PROGRESS_GTCLAUSE | X1f4_PROGRESS_LTCLAUSE

static const struct x1f4_position_type position_1x[] = {
/* *INDENT-OFF* */
    {
    {	"b_jack(;1, ;*, ;0)",	1,	1,	0,	1,	18	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"bt_next(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"bt_previous(;1, ;*, ;0)",
				1,	1,	0,	1,	23	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
};
static const struct x1f4_progress_type progress_1b = {
/* *INDENT-OFF* */
    {	"bt_previous(;1, ;*, ;0)",
				1,	1,	0,	1,	23	},
    {	"bt_last(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_E4_MODE,	X1f4_E4_MODE,	0,
	X1f4_PROGRESS_BACKWARD | DATA_PROGRESS_POSITION,	2,
	position_1x
/* *INDENT-ON* */
}, progress_1f = {
/* *INDENT-OFF* */
    {	"bt_next(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
    {	"bt_first(;1, ;*, ;0)",	1,	1,	0,	1,	20	},
	X1f4_E4_MODE,	X1f4_E4_MODE,	0,
	DATA_PROGRESS_POSITION,		2,
	position_1x
/* *INDENT-ON* */
};

int
x1f4_pass_lxdata(void *lxdata, struct x1f4_progress_type **progress)
{
    int type;
    struct x1f4_progress_type *progress_data;

    type = ((struct lxdata_type *) lxdata)->link_t.datatype[0].type;

    progress_data = *progress;

    *progress_data = progress_1f;
    progress_data->type = type;
    progress_data++;

    *progress_data = progress_1b;
    progress_data->type = type;
    progress_data++;

    *progress = progress_data;

    return 0;
}

