/*
 * lxdecq-a.0.c
 * Copyright (C) 2008-2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * NOTES
 *
 *.no type generic interfaces (yet).  If/when added, they should account for
 * function pointers.
 */

#include <lxdecq-config.h>

#include <stddef.h>
#include <string.h>

#include <e4-m.0.h>
#include <e4.h>
#include <f_list.h>
#include <lxcall.h>
#include <lxdecq-defs.h>
#include <lxdecq-inter.h>
#include <lxdecq-types.h>
#include <lxnext.h>

#define MAKE_SINGLE(a, b)		a

#define may_include(lxtype)		0

#define lbe_type_post			2

#define lbg_type_post			2

#define lbgstype_post			2

#define lbj_type_post			2

#define lbjstype_post			2

#define lbo_type_post			2

#define lfe_type_post			2

#define lfg_type_post			2

#define lfgstype_post			2

#define lfj_type_post			2

#define lfjstype_post			2

#define lfo_type_post			2

#define true(e)				1

#define screen(screen) \
    ((struct screen_type *) (screen))

#define fb_size()			1
#define fp_size() \
    (sizeof(side_u) / sizeof(struct x1f4_operator_type))

#define fm_size(none) \
    sizeof(struct lxmiss_type)

extern const struct x1f4_eelookup_type _x1f4_type_lookup;

static int init_list(struct lxdecq_type *, unsigned,
		     const struct screen_type *);
static int line_data(struct lxdecq_type *, unsigned,
		     const struct screen_type *, void *, void *, void **,
		     void **);
static int line_list(struct lxdecq_type *, unsigned,
		     const struct screen_type *);
static int line_plus(struct lxdecq_type *);
static int line_sail(struct lxdecq_type *, const struct screen_type *,
		     struct lxtext_type *, const struct lxtype_type *);
static int line_text(struct lxdecq_type *, unsigned,
		     const struct screen_type *, void *, void **, void **);
static int link_list(struct lxdecq_type *, unsigned,
		     const struct screen_type *);
static int pick_miss(unsigned *, unsigned *, unsigned *, unsigned,
		     const struct screen_type *);

static void borne_n_l(v_case_args_____0);
static void borneo__l(u_case_args_____0);
static void borneo_ln(v_case_args_____0);
static void born_slx0(u_case_args_____0);
static void line_epix(v_case_args_____0);
static void line_esix(v_case_args_____0);
static void line_gpix(v_case_args_____0);
static void line_gsix(v_case_args_____0);
static void line_gsup(v_case_args_____0);
static void line_gsus(v_case_args_____0);
static void line_jpix(v_case_args_____0);
static void line_jsix(v_case_args_____0);
static void line_jsup(v_case_args_____0);
static void line_jsus(v_case_args_____0);
static void line_nset(u_case_args_____0);
static void line_opix(v_case_args_____0);
static void line_osix(v_case_args_____0);
static void line_ppix(v_case_args_____0);
static void line_psix(v_case_args_____0);
static void line_qpix(v_case_args_____0);
static void line_qsix(v_case_args_____0);
static void line_qsup(v_case_args_____0);
static void line_qsus(v_case_args_____0);
static void line_sset(u_case_args_____0);
static void line_xpix(v_case_args_____0);
static void line_xsix(v_case_args_____0);
static void line_yfix(u_case_args_____0);
static void themed__l(u_case_args_____0);
static void themed_ll(u_case_args_____0);

static const char fq_lengthf[] = "q_length", fq_setf[] = "q_set",
    *const side_f[] = {
/* *INDENT-OFF* */
    fq_setf
/* *INDENT-ON* */
}, *const side_l[] = {
/* *INDENT-OFF* */
    fq_lengthf
/* *INDENT-ON* */
};
static int (*const rate_epix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_e0class_b,
    _libx1f4i0_lxdecq_e0class_m,
    _libx1f4i0_lxdecq_e0class_r,
    _libx1f4i0_lxdecq_e0class_t,
    _libx1f4i0_lxdecq_ebclass_u,
    _libx1f4i0_lxdecq_ebclass_v
/* *INDENT-ON* */
}, (*const rate_esix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_e0class_b,
    _libx1f4i0_lxdecq_e0class_m,
    _libx1f4i0_lxdecq_e0class_r,
    _libx1f4i0_lxdecq_e0class_t,
    _libx1f4i0_lxdecq_efclass_u,
    _libx1f4i0_lxdecq_efclass_v
/* *INDENT-ON* */
}, (*const rate_gpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_g0class_b,
    _libx1f4i0_lxdecq_g0class_m,
    _libx1f4i0_lxdecq_g0class_r,
    _libx1f4i0_lxdecq_g0class_t,
    _libx1f4i0_lxdecq_gbclass_u,
    _libx1f4i0_lxdecq_gbclass_v
/* *INDENT-ON* */
}, (*const rate_gsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_g0class_b,
    _libx1f4i0_lxdecq_g0class_m,
    _libx1f4i0_lxdecq_g0class_r,
    _libx1f4i0_lxdecq_g0class_t,
    _libx1f4i0_lxdecq_gfclass_u,
    _libx1f4i0_lxdecq_gfclass_v
/* *INDENT-ON* */
}, (*const rate_gsup[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_g0classsb,
    _libx1f4i0_lxdecq_g0classsm,
    _libx1f4i0_lxdecq_g0classsr,
    _libx1f4i0_lxdecq_g0classst,
    _libx1f4i0_lxdecq_gbclasssu,
    _libx1f4i0_lxdecq_gbclasssv
/* *INDENT-ON* */
}, (*const rate_gsus[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_g0classsb,
    _libx1f4i0_lxdecq_g0classsm,
    _libx1f4i0_lxdecq_g0classsr,
    _libx1f4i0_lxdecq_g0classst,
    _libx1f4i0_lxdecq_gfclasssu,
    _libx1f4i0_lxdecq_gfclasssv
/* *INDENT-ON* */
}, (*const rate_jpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_j0class_b,
    _libx1f4i0_lxdecq_j0class_m,
    _libx1f4i0_lxdecq_j0class_r,
    _libx1f4i0_lxdecq_j0class_t,
    _libx1f4i0_lxdecq_jbclass_u,
    _libx1f4i0_lxdecq_jbclass_v
/* *INDENT-ON* */
}, (*const rate_jsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_j0class_b,
    _libx1f4i0_lxdecq_j0class_m,
    _libx1f4i0_lxdecq_j0class_r,
    _libx1f4i0_lxdecq_j0class_t,
    _libx1f4i0_lxdecq_jfclass_u,
    _libx1f4i0_lxdecq_jfclass_v
/* *INDENT-ON* */
}, (*const rate_jsup[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_j0classsb,
    _libx1f4i0_lxdecq_j0classsm,
    _libx1f4i0_lxdecq_j0classsr,
    _libx1f4i0_lxdecq_j0classst,
    _libx1f4i0_lxdecq_jbclasssu,
    _libx1f4i0_lxdecq_jbclasssv
/* *INDENT-ON* */
}, (*const rate_jsus[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_j0classsb,
    _libx1f4i0_lxdecq_j0classsm,
    _libx1f4i0_lxdecq_j0classsr,
    _libx1f4i0_lxdecq_j0classst,
    _libx1f4i0_lxdecq_jfclasssu,
    _libx1f4i0_lxdecq_jfclasssv
/* *INDENT-ON* */
}, (*const rate_opix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_o0class_b,
    _libx1f4i0_lxdecq_o0class_m,
    _libx1f4i0_lxdecq_o0class_r,
    _libx1f4i0_lxdecq_o0class_t,
    _libx1f4i0_lxdecq_obclass_u,
    _libx1f4i0_lxdecq_obclass_v
/* *INDENT-ON* */
}, (*const rate_osix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_o0class_b,
    _libx1f4i0_lxdecq_o0class_m,
    _libx1f4i0_lxdecq_o0class_r,
    _libx1f4i0_lxdecq_o0class_t,
    _libx1f4i0_lxdecq_ofclass_u,
    _libx1f4i0_lxdecq_ofclass_v
/* *INDENT-ON* */
}, (*const rate_qpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_q0class_b,
    _libx1f4i0_lxdecq_q0class_m,
    _libx1f4i0_lxdecq_q0class_r,
    _libx1f4i0_lxdecq_q0class_t,
    _libx1f4i0_lxdecq_qbclass_u,
    _libx1f4i0_lxdecq_qbclass_v
/* *INDENT-ON* */
}, (*const rate_qsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_q0class_b,
    _libx1f4i0_lxdecq_q0class_m,
    _libx1f4i0_lxdecq_q0class_r,
    _libx1f4i0_lxdecq_q0class_t,
    _libx1f4i0_lxdecq_qfclass_u,
    _libx1f4i0_lxdecq_qfclass_v
/* *INDENT-ON* */
}, (*const rate_qsup[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_q0classsb,
    _libx1f4i0_lxdecq_q0classsm,
    _libx1f4i0_lxdecq_q0classsr,
    _libx1f4i0_lxdecq_q0classst,
    _libx1f4i0_lxdecq_qbclasssu,
    _libx1f4i0_lxdecq_qbclasssv
/* *INDENT-ON* */
}, (*const rate_qsus[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_q0classsb,
    _libx1f4i0_lxdecq_q0classsm,
    _libx1f4i0_lxdecq_q0classsr,
    _libx1f4i0_lxdecq_q0classst,
    _libx1f4i0_lxdecq_qfclasssu,
    _libx1f4i0_lxdecq_qfclasssv
/* *INDENT-ON* */
}, (*const rate_xpix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_x0class_b,
    _libx1f4i0_lxdecq_x0class_m,
    _libx1f4i0_lxdecq_x0class_r,
    _libx1f4i0_lxdecq_x0class_t,
    _libx1f4i0_lxdecq_xbclass_u,
    _libx1f4i0_lxdecq_xbclass_v
/* *INDENT-ON* */
}, (*const rate_xsix[])(void *, void *, void **) = {
/* *INDENT-OFF* */
    _libx1f4i0_lxdecq_x0class_b,
    _libx1f4i0_lxdecq_x0class_m,
    _libx1f4i0_lxdecq_x0class_r,
    _libx1f4i0_lxdecq_x0class_t,
    _libx1f4i0_lxdecq_xfclass_u,
    _libx1f4i0_lxdecq_xfclass_v
/* *INDENT-ON* */
};
static const struct line_type auto_line[] = {
/* *INDENT-OFF* */
#define decq_count			0
    {	{	"decq",
		_libx1f4i0_lxdecq_y_logique,	0,
		NULL,				decq_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						4		},
	line_yfix,							},
#define decq_reach			(0 + decq_count)
#define l_new_count			1
    {	{	"q_new",
		_libx1f4i0_lxdecq_n_forward,	X1f4_E4_VOID,
		NULL,				l_new_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,
						5		},
	line_nset,							},
#define l_new_reach			(decq_reach + (l_new_count << 1))
#define l_set_count			2
    {	{	fq_setf,
		_libx1f4i0_lxdecq_s_forward,	0,
		NULL,				l_set_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_TEXT_LINK,
						5		},
	line_sset,							}
#define l_set_reach			(l_new_reach + (l_set_count << 1))
/* *INDENT-ON* */
#define auto_line_count			l_set_reach
}, gray_line[] = {
/* *INDENT-OFF* */
#define l_roll_count			1
    {	{	"q_roll",
		_libx1f4i0_lxdecq_f_keeping,	0,
		NULL,				l_roll_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed__l,							},
#define l_roll_reach			(0 + l_roll_count)
#define l_short_count			1
    {	{	"q_short",
		_libx1f4i0_lxdecq_a_keeping,	0,
		NULL,				l_short_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define l_short_reach			(l_roll_reach + l_short_count)
#define lb_delete_count			1
    {	{	"qb_delete",
		_libx1f4i0_lxdecq_a_keeping,	0,
		NULL,				lb_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define lb_delete_reach			(l_short_reach + lb_delete_count)
/* *INDENT-ON* */
#define gray_line_count			lb_delete_reach
}, head_line[] = {
/* *INDENT-OFF* */
#define lf_jack_count			2
    {	{	"qf_jack_s",
		_libx1f4i0_lxdecq_jskeeping,	X1f4_E4_MODE,
		NULL,				lf_jack_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	born_slx0,							}
#define lf_jack_reach			(0 + (lf_jack_count << 1))
/* *INDENT-ON* */
#define head_line_count			lf_jack_reach
}, slip_line[] = {
/* *INDENT-OFF* */
#define l_cast_count			2
    {	{	"q_cast",
		_libx1f4i0_lxdecq_c_nearest,	0,
		NULL,				l_cast_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define l_cast_reach			(0 + l_cast_count)
#define l_clear_count			1
    {	{	"q_clear",
		_libx1f4i0_lxdecq_c_pending,	0,
		NULL,				l_clear_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define l_clear_reach			(l_cast_reach + l_clear_count)
#define l_copy_count			2
    {	{	"q_copy",
		_libx1f4i0_lxdecq_c_logique,	0,
		NULL,				l_copy_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							},
#define l_copy_reach			(l_clear_reach + l_copy_count)
#define l_length_count			1
    {	{	fq_lengthf,
		_libx1f4i0_lxdecq_l_logique,	X1f4_E4_MODE,
		NULL,				l_length_count,
		0,
						8		},
	borneo__l,							},
#define l_length_reach			(l_copy_reach + l_length_count)
#define l_swap_count			2
    {	{	"q_swap",
		_libx1f4i0_lxdecq_s_logique,	X1f4_E4_VOID,
		NULL,				l_swap_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed_ll,							}
#define l_swap_reach			(l_length_reach + l_swap_count)
/* *INDENT-ON* */
#define slip_line_count			l_swap_reach
}, tail_line[] = {
/* *INDENT-OFF* */
#define lb_jack_count			2
    {	{	"qb_jack_s",
		_libx1f4i0_lxdecq_jskeeping,	X1f4_E4_MODE,
		NULL,				lb_jack_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE
		| X1f4_E4_SLIP_LIST | X1f4_E4_TEXT_LINK,
						9		},
	born_slx0,							}
#define lb_jack_reach			(0 + (lb_jack_count << 1))
/* *INDENT-ON* */
#define tail_line_count			lb_jack_reach
}, used_line[] = {
/* *INDENT-OFF* */
#define l_class_count			1
    {	{	"q_class",
		_libx1f4i0_lxdecq_a_keeping,	0,
		NULL,				l_class_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						7		},
	themed__l,							},
#define l_class_reach			(0 + l_class_count)
#define l_reel_count			1
    {	{	"q_reel",
		_libx1f4i0_lxdecq_f_keeping,	0,
		NULL,				l_reel_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						6		},
	themed__l,							},
#define l_reel_reach			(l_class_reach + l_reel_count)
#define lf_delete_count			1
    {	{	"qf_delete",
		_libx1f4i0_lxdecq_a_keeping,	0,
		NULL,				lf_delete_count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						9		},
	themed__l,							},
#define lf_delete_reach			(l_reel_reach + lf_delete_count)
/* *INDENT-ON* */
#define used_line_count			lf_delete_reach
};
static const struct orient_type cast_line[] = {
/* *INDENT-OFF* */
#define lb_q__count			1
    {	{	"qb_q_",
		NULL,				0,
		NULL,				lb_q__count,
		X1f4_E4_TEXT_LINK,		5		},
	line_qpix,							},
#define lb_q__reach			(0 + lb_q__count)
#define lf_q__count			1
    {	{	"qf_q_",
		NULL,				0,
		NULL,				lf_q__count,
		X1f4_E4_TEXT_LINK,		5		},
	line_qsix,							}
#define lf_q__reach			(lb_q__reach + lf_q__count)
/* *INDENT-ON* */
#define cast_line_count			lf_q__reach
}, east_line[] = {
/* *INDENT-OFF* */
#define lb_qs__count			1
    {	{	"qb_qs_",
		NULL,				0,
		NULL,				lb_qs__count,
		X1f4_E4_TEXT_LINK,		6		},
	line_qsup,							},
#define lb_qs__reach			(0 + lb_qs__count)
#define lf_qs__count			1
    {	{	"qf_qs_",
		NULL,				0,
		NULL,				lf_qs__count,
		X1f4_E4_TEXT_LINK,		6		},
	line_qsus,							}
#define lf_qs__reach			(lb_qs__reach + lf_qs__count)
/* *INDENT-ON* */
#define east_line_count			lf_qs__reach
}, fast_line[] = {
/* *INDENT-OFF* */
#define lb_e__count			2
    {	{	"qb_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				lb_e__count,
		0,				5		},
	line_epix,							},
#define lb_e__reach			(0 + lb_e__count)
#define lb_g__count			2
    {	{	"qb_g_",
		NULL,				0,
		NULL,				lb_g__count,
		0,				5		},
	line_gpix,							},
#define lb_g__reach			(lb_e__reach + lb_g__count)
#define lb_j__count			2
    {	{	"qb_j_",
		NULL,				X1f4_E4_MODE,
		NULL,				lb_j__count,
		0,				5		},
	line_jpix,							},
#define lb_j__reach			(lb_g__reach + lb_j__count)
#define lb_o__count			2
    {	{	"qb_o_",
		NULL,				X1f4_E4_MODE,
		NULL,				lb_o__count,
		0,				5		},
	line_opix,							},
#define lb_o__reach			(lb_j__reach + lb_o__count)
#define lb_p__count			2
    {	{	"qb_p_",
		NULL,				X1f4_E4_VOID,
		NULL,				lb_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	line_ppix,							},
#define lb_p__reach			(lb_o__reach + lb_p__count)
#define lf_e__count			2
    {	{	"qf_e_",
		NULL,				X1f4_E4_VOID,
		NULL,				lf_e__count,
		0,				5		},
	line_esix,							},
#define lf_e__reach			(lb_p__reach + lf_e__count)
#define lf_g__count			2
    {	{	"qf_g_",
		NULL,				0,
		NULL,				lf_g__count,
		0,				5		},
	line_gsix,							},
#define lf_g__reach			(lf_e__reach + lf_g__count)
#define lf_j__count			2
    {	{	"qf_j_",
		NULL,				X1f4_E4_MODE,
		NULL,				lf_j__count,
		0,				5		},
	line_jsix,							},
#define lf_j__reach			(lf_g__reach + lf_j__count)
#define lf_o__count			2
    {	{	"qf_o_",
		NULL,				X1f4_E4_MODE,
		NULL,				lf_o__count,
		0,				5		},
	line_osix,							},
#define lf_o__reach			(lf_j__reach + lf_o__count)
#define lf_p__count			2
    {	{	"qf_p_",
		NULL,				X1f4_E4_VOID,
		NULL,				lf_p__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	line_psix,							}
#define lf_p__reach			(lf_o__reach + lf_p__count)
/* *INDENT-ON* */
#define fast_line_count			lf_p__reach
}, last_line[] = {
/* *INDENT-OFF* */
#define lb_gs__count			2
    {	{	"qb_gs_",
		NULL,				0,
		NULL,				lb_gs__count,
		0,				6		},
	line_gsup,							},
#define lb_gs__reach			(0 + lb_gs__count)
#define lb_js__count			2
    {	{	"qb_js_",
		NULL,				X1f4_E4_MODE,
		NULL,				lb_js__count,
		0,				6		},
	line_jsup,							},
#define lb_js__reach			(lb_gs__reach + lb_js__count)
#define lf_gs__count			2
    {	{	"qf_gs_",
		NULL,				0,
		NULL,				lf_gs__count,
		0,				6		},
	line_gsus,							},
#define lf_gs__reach			(lb_js__reach + lf_gs__count)
#define lf_js__count			2
    {	{	"qf_js_",
		NULL,				X1f4_E4_MODE,
		NULL,				lf_js__count,
		0,				6		},
	line_jsus,							}
#define lf_js__reach			(lf_gs__reach + lf_js__count)
/* *INDENT-ON* */
#define last_line_count			lf_js__reach
}, long_line[] = {
/* *INDENT-OFF* */
#define lb_l__count			2
    {	{	"qb_l_",
		_libx1f4i0_lxdecq_u_forward,	X1f4_E4_VOID,
		NULL,				lb_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo_ln,							},
#define lb_l__reach			(0 + lb_l__count)
#define lf_l__count			2
    {	{	"qf_l_",
		_libx1f4i0_lxdecq_c_forward,	X1f4_E4_VOID,
		NULL,				lf_l__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borneo_ln,							}
#define lf_l__reach			(lb_l__reach + lf_l__count)
/* *INDENT-ON* */
#define long_line_count			lf_l__reach
}, past_line[] = {
/* *INDENT-OFF* */
#define lb_x__count			1
    {	{	"qb_x_",
		NULL,				0,
		NULL,				lb_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	line_xpix,							},
#define lb_x__reach			(0 + lb_x__count)
#define lf_x__count			1
    {	{	"qf_x_",
		NULL,				0,
		NULL,				lf_x__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	line_xsix,							}
#define lf_x__reach			(lb_x__reach + lf_x__count)
/* *INDENT-ON* */
#define past_line_count			lf_x__reach
}, spin_line[] = {
/* *INDENT-OFF* */
#define l_f__count			1
    {	{	"qf_n_",
		_libx1f4i0_lxdecq_f_forward,	0,
		NULL,				l_f__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borne_n_l,							},
#define l_f__reach			(0 + l_f__count)
#define l_z__count			1
    {	{	"qb_n_",
		_libx1f4i0_lxdecq_z_forward,	0,
		NULL,				l_z__count,
		X1f4_E4_KEEP_CALL | X1f4_E4_TEXT_LINK,
						5		},
	borne_n_l,							}
#define l_z__reach			(l_f__reach + l_z__count)
/* *INDENT-ON* */
#define spin_line_count			l_z__reach
};
static const struct x1f4_operator_type side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),  NULL,		0400,
	0,			NULL,
	X1f4_E4_BACK_LINK | X1f4_E4_E2ND_LINK | X1f4_E4_LEFT_XSET,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
}, side_u[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("~", " "),  NULL,		0210,
	0,			NULL,
	X1f4_E4_E2ND_LINK,
				1,
	NULL,			NULL					}
/* *INDENT-ON* */
};

static int
init_list(struct lxdecq_type *lxdecq_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;

    if (bits & AUTOLINK_LINK) {
	lxdecq_data->link_v = screen_data->lock_v;
    } else {
	lxdecq_data->link_v.data = NULL;
	lxdecq_data->link_v.free = NULL;
	lxdecq_data->link_v.link = NULL;
	lxdecq_data->link_v.mode = NULL;
	lxdecq_data->link_v.pick = NULL;
	lxdecq_data->link_v.slip = NULL;
    }

    if (bits & CODELINK_LINK) {
	lxdecq_data->link_w = screen_data->lock_w;
    } else {
	lxdecq_data->link_w.data = NULL;
	lxdecq_data->link_w.free = _x1f4_e4_free_data;
	lxdecq_data->link_w.link = _x1f4_e4_link_data;
	lxdecq_data->link_w.mode = _x1f4_e4_mode_data;
    }

    if (bits & EELOOKUP_LINK) {
	lxdecq_data->trap_e.eelookup_data = screen_data->lock_g.eelookup_data;
    } else {
	lxdecq_data->trap_e.eelookup_data = &_x1f4_type_lookup;
    }

    if (bits & MISSBAIL_LINK) {
	lxdecq_data->link_r.call = screen_data->lock_r.call;
	lxdecq_data->link_r.fine = screen_data->lock_r.fine;
	lxdecq_data->link_r.miss = screen_data->lock_r.miss;
	lxdecq_data->link_r.text = screen_data->lock_r.text;
    } else {
	lxdecq_data->link_r.call = NULL;
	lxdecq_data->link_r.fine = NULL;
	lxdecq_data->link_r.miss = NULL;
    }

    if (bits & RESOURCE_LINK) {
	lxdecq_data->link_m = screen_data->lock_m;
    } else {
	lxdecq_data->link_m.data = NULL;
	lxdecq_data->link_m.free = _x1f4_e4_free_data;
	lxdecq_data->link_m.link = _x1f4_e4_link_data;
	lxdecq_data->link_m.mode = _x1f4_e4_mode_data;
    }

    if (bits & TEXTFLAT_LINK) {
	lxdecq_data->link_e = screen_data->lock_e;
    } else {
	lxdecq_data->link_e.data = NULL;
	lxdecq_data->link_e.line = NULL;
	lxdecq_data->link_e.post = NULL;
	lxdecq_data->link_e.push = NULL;
    }

    status = link_list(lxdecq_data, bits, screen_data);

    return status;
}


static int
line_data(struct lxdecq_type *lxdecq_data, unsigned bits,
	  const struct screen_type *screen_data, void *data, void *text,
	  void **args, void **dana)
{
    struct lxtext_type *lxtext_data;
    const struct lxtype_type *lxtype_data;
    struct x1f4_linetext_type *linetext_data;
    unsigned i;

    lxtype_data = screen_data->lock_t.data;

    linetext_data = data;
    lxtext_data = text;

    lxtype_data--;

    /*
     * place holder for _af_
     */
    lxtext_data->lxtype.flags = X1f4_LX_TYPE_EXCEPT;

    i = screen_data->lock_t.miss;
    for (; i; i--) {
	unsigned flags;

	lxtext_data++;

	lxtype_data++;

	line_sail(lxdecq_data, screen_data, lxtext_data, lxtype_data);

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    continue;
	}

	if (1) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(fast_line) / sizeof(struct orient_type),
		 fast_line);
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(last_line) / sizeof(struct orient_type),
		 last_line);
	}
	if (bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(cast_line) / sizeof(struct orient_type),
		 cast_line);
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(east_line) / sizeof(struct orient_type),
		 east_line);
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(past_line) / sizeof(struct orient_type),
		 past_line);
	}

	if (flags & LINK_ACCESS) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(long_line) / sizeof(struct orient_type),
		 long_line);
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    _libx1f4i0_lxcall_push_line
		(lxtext_data, bits, screen_data, &linetext_data, lxtype_data,
		 args, dana, sizeof(spin_line) / sizeof(struct orient_type),
		 spin_line);
	}
    }

    return 0;
}


static int
line_list(struct lxdecq_type *lxdecq_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    lxdecq_data->link_t.datatype[0].context = lxdecq_data;

    lxdecq_data->link_t.datatype[0].flat = _libx1f4i0_lxdecq_flat_list;
    lxdecq_data->link_t.datatype[0].lead = _libx1f4i0_lxdecq_lead_list;
    lxdecq_data->link_t.datatype[0].line = _libx1f4i0_lxdecq_line_list;
    lxdecq_data->link_t.datatype[0].link = _libx1f4i0_lxdecq_link_list;
    lxdecq_data->link_t.datatype[0].name = "decq";
    lxdecq_data->link_t.datatype[0].slip = _libx1f4i0_lxdecq_slip_list;
    lxdecq_data->link_t.datatype[0].size = 4;
    lxdecq_data->link_t.datatype[0].type = screen_data->link_l.type;

    lxdecq_data->link_s.nodetype[0].context = lxdecq_data;

    _libx1f4i0_lxdecq_type_list
	(lxdecq_data->link_s.nodetype + 0, screen_data->link_l.type);

    lxdecq_data->link_t.datatype[1].context = NULL;

    lxdecq_data->link_t.datatype[1].flat = NULL;
    lxdecq_data->link_t.datatype[1].lead = NULL;
    lxdecq_data->link_t.datatype[1].line = NULL;
    lxdecq_data->link_t.datatype[1].link = NULL;
    lxdecq_data->link_t.datatype[1].name = NULL;
    lxdecq_data->link_t.datatype[1].slip = NULL;
    lxdecq_data->link_t.datatype[1].size = 0;
    lxdecq_data->link_t.datatype[1].type = 0;

    return 0;
}


static int
line_plus(struct lxdecq_type *lxdecq_data)
{
    struct cxbend_type *cxbend_data;

    cxbend_data = lxdecq_data->rule_f.cxbend;

    cxbend_data[0].bend = 0;
    cxbend_data[0].list = lxdecq_data;
    cxbend_data[0].pick = _libx1f4i0_lead_dllist;
    cxbend_data[0].push = _libx1f4i0_head_dllist;
    cxbend_data[0].roll = _libx1f4i0_reel_dllist;
    cxbend_data[0].slip = _libx1f4i0_mind_dllist;

    cxbend_data++;

    cxbend_data[0].bend = 1;
    cxbend_data[0].list = lxdecq_data;
    cxbend_data[0].pick = _libx1f4i0_ever_dllist;
    cxbend_data[0].push = _libx1f4i0_tail_dllist;
    cxbend_data[0].roll = _libx1f4i0_roll_dllist;
    cxbend_data[0].slip = _libx1f4i0_slip_dllist;

    return 0;
}


static int
line_sail(struct lxdecq_type *lxdecq_data,
	  const struct screen_type *screen_data,
	  struct lxtext_type *lxtext_data,
	  const struct lxtype_type *lxtype_data)
{
    int code;

    code = lxtype_data->code;

    lxtext_data->lxtype = *lxtype_data;
    lxtext_data->text = lxdecq_data;

    if (0) {
    } else {
	if (code == X1f4_E4_TEXT) {
	    lxtext_data->lxtype.context = &lxdecq_data->ever_a;

	    lxtext_data->lxtype.copy = _libx1f4i0_lxnext_copy_text;
	    lxtext_data->lxtype.free = _libx1f4i0_lxnext_free_text;
	    lxtext_data->lxtype.lead = _libx1f4i0_lxnext_lead_text;
	    lxtext_data->lxtype.move = NULL;
	    lxtext_data->lxtype.push = _libx1f4i0_lxnext_push_text;
	    lxtext_data->lxtype.slip = _libx1f4i0_lxnext_slip_text;
	}
    }

    return 0;
}


static int
line_text(struct lxdecq_type *lxdecq_data, unsigned bits,
	  const struct screen_type *screen_data, void *data, void **args,
	  void **dana)
{
#define cxbend_ever			lxdecq_data->rule_f.cxbend + 1
#define cxbend_lead			lxdecq_data->rule_f.cxbend + 0

    struct x1f4_linetext_type *linetext_data;

    linetext_data = data;

    if (1) {
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, screen_data, &linetext_data, args,
	     sizeof(gray_line) / sizeof(struct line_type), gray_line);
	_libx1f4i0_lxcall_side_line
	    (lxdecq_data, bits, screen_data, &linetext_data, args,
	     sizeof(slip_line) / sizeof(struct line_type), slip_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, screen_data, &linetext_data, args,
	     sizeof(used_line) / sizeof(struct line_type), used_line);
    }
    if (bits & AUTOLINK_LINK) {
	_libx1f4i0_lxcall_side_line
	    (lxdecq_data, bits, screen_data, &linetext_data, args,
	     sizeof(auto_line) / sizeof(struct line_type), auto_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_lead, bits, screen_data, &linetext_data, args,
	     sizeof(head_line) / sizeof(struct line_type), head_line);
	_libx1f4i0_lxcall_side_line
	    (cxbend_ever, bits, screen_data, &linetext_data, args,
	     sizeof(tail_line) / sizeof(struct line_type), tail_line);
    }
    if (1) {
	_libx1f4i0_lxcall_zero_line(linetext_data);
    }

    return 0;

#undef cxbend_lead
#undef cxbend_ever
}


static int
link_list(struct lxdecq_type *lxdecq_data, unsigned bits,
	  const struct screen_type *screen_data)
{
    int status;
    unsigned args_class, args_count, line_class, line_count, main_class,
	main_count, miss;
    void *data;

    args_count = 0;
    line_count = 1;
    main_count = 0;

    pick_miss(&args_class, &line_class, &main_class, bits, screen_data);

    miss = screen_data->lock_t.miss;

    {
	args_count += gray_line_count;
	line_count += sizeof(gray_line) / sizeof(struct line_type);
	args_count += slip_line_count;
	line_count += sizeof(slip_line) / sizeof(struct line_type);
	args_count += used_line_count;
	line_count += sizeof(used_line) / sizeof(struct line_type);
    }
    if (bits & AUTOLINK_LINK) {
	args_count += auto_line_count;
	line_count += sizeof(auto_line) / sizeof(struct line_type);
	args_count += head_line_count;
	line_count += sizeof(head_line) / sizeof(struct line_type);
	args_count += tail_line_count;
	line_count += sizeof(tail_line) / sizeof(struct line_type);
    }

    status = lxdecq_data->link_w.link
	(lxdecq_data->link_w.data, &data,
	 fm_size(away)
	 + (args_class + args_count) * sizeof(int)
	 + (1 + miss) * sizeof(struct lxtext_type)
	 + (line_class + line_count) * sizeof(struct x1f4_linetext_type)
	 + (sizeof(struct ticket_type) + sizeof(struct x1f4_operator_type *))
	 * (fb_size() + fp_size()) + sizeof(struct x1f4_operator_type *) * 2
	 + main_class + main_count);
    if (status) {
	status = LINK_ERROR;
    } else {
	void *args, *dana, *rate, *side, *tile, *text;

	tile = (struct x1f4_linetext_type *) data + line_class;
	dana = (struct x1f4_linetext_type *) tile + line_count;
	text = (struct lxmiss_type *) dana + 1;
	side = (struct lxtext_type *) text + miss + 1;
	side = (struct ticket_type *) side + fb_size();
	rate = (struct x1f4_operator_type **) side + fb_size() + 1;
	rate = (struct ticket_type *) rate + fp_size();
	args = (struct x1f4_operator_type **) rate + fp_size() + 1;
	dana = (int *) args + args_class + args_count;

	lxdecq_data->link_f.data = data;
	lxdecq_data->link_f.rate = rate;
	lxdecq_data->link_f.side = side;
	lxdecq_data->link_f.text = text;

	((struct lxmiss_type *) text - 1)->data.miss = miss;

        line_plus(lxdecq_data);

	line_data(lxdecq_data, bits, screen_data, data, text, &args, &dana);
	line_text(lxdecq_data, bits, screen_data, tile, &args, &dana);
	_libx1f4i0_lxcall_line_side(side, data, side_o, side_f, fb_size());
	_libx1f4i0_lxcall_line_side(rate, data, side_u, side_l, fp_size());

	line_list(lxdecq_data, bits, screen_data);
    }

    return status;
}


static int
pick_miss(unsigned *args, unsigned *line, unsigned *pick, unsigned bits,
	  const struct screen_type *screen_data)
{
    const struct lxtype_type *lxtype_data;
    unsigned args_count = 0, line_count = 0, i, main_count = 0;

    lxtype_data = screen_data->lock_t.data;
    lxtype_data--;

    i = screen_data->lock_t.miss;
    for (; i; i--) {
	unsigned deck, flags;

	lxtype_data++;

	flags = lxtype_data->flags;

	if (flags & X1f4_LX_TYPE_EXCEPT) {
	    continue;
	}

	deck = 5 + lxtype_data->size + 1;

	if (1) {
	    args_count += fast_line_count;
	    line_count += sizeof(fast_line) / sizeof(struct orient_type);
	    main_count +=
		deck * sizeof(fast_line) / sizeof(struct orient_type);
	    args_count += last_line_count;
	    line_count += sizeof(last_line) / sizeof(struct orient_type);
	    main_count +=
		(deck + 1) * sizeof(last_line) / sizeof(struct orient_type);
	}
	if (bits & AUTOLINK_LINK) {
	    args_count += cast_line_count;
	    line_count += sizeof(cast_line) / sizeof(struct orient_type);
	    main_count +=
		deck * sizeof(cast_line) / sizeof(struct orient_type);
	    args_count += cast_line_count;
	    line_count += sizeof(east_line) / sizeof(struct orient_type);
	    main_count +=
		(deck + 1) * sizeof(east_line) / sizeof(struct orient_type);
	}

	if (flags & FLAT_REFLEX) {
	} else {
	    args_count += lbe_type_post;
	    args_count += lbg_type_post;
	    args_count += lbgstype_post;
	    args_count += lbj_type_post;
	    args_count += lbjstype_post;
	    args_count += lbo_type_post;
	    args_count += lfe_type_post;
	    args_count += lfg_type_post;
	    args_count += lfgstype_post;
	    args_count += lfj_type_post;
	    args_count += lfjstype_post;
	    args_count += lfo_type_post;
	}
	if (flags & LINK_ACCESS) {
	    args_count += long_line_count;
	    line_count += sizeof(long_line) / sizeof(struct orient_type);
	    main_count +=
		deck * sizeof(long_line) / sizeof(struct orient_type);
	}
	if (flags & POST_ACCESS && bits & AUTOLINK_LINK) {
	    args_count += spin_line_count;
	    line_count += sizeof(spin_line) / sizeof(struct orient_type);
	    main_count +=
		deck * sizeof(spin_line) / sizeof(struct orient_type);
	}

	if (bits & AUTOLINK_LINK
	    || (!lxtype_data->lead && true(code != X1f4_E4_TEXT))) {
	    args_count += past_line_count;
	    line_count += sizeof(past_line) / sizeof(struct orient_type);
	    main_count +=
		deck * sizeof(past_line) / sizeof(struct orient_type);
	}
    }

    *args = args_count;
    *line = line_count;
    *pick = main_count;

    return 0;
}


static void
borne_n_l(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = nodetype_data->code;
}


static void
borneo__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;
}


static void
borneo_ln(v_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = nodetype_data->code;

    *args = line;
}


static void
born_slx0(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = X1f4_E4_SLIP;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;
}



static void
line_epix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_epix);
}


static void
line_esix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_esix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_esix);
}


static void
line_gpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_gpix);
}


static void
line_gsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_gsix);
}


static void
line_gsup(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_gsup);
}


static void
line_gsus(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_gsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_gsus);
}


static void
line_jpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_jsix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_jpix);
}


static void
line_jsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_jsix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_jsix);
}


static void
line_jsup(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_jsix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_jsup);
}


static void
line_jsus(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_jsix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_jsus);
}


static void
line_nset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_XSET;

    *args = line;
}


static void
line_opix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_osix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_opix);
}


static void
line_osix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_osix
	(v_case_args_____2, screen(screen)->link_l.type, X1f4_E4_MODE,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_osix);
}


static void
line_ppix(v_case_args_____1)
{
    int *line, type;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = type;

    *args = line;

    if (nodetype_data->copy || may_include(lxtype_data)) {
	linetext_data->function.function = _libx1f4i0_lxdecq_pbclass_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxdecq_pbclass_u;
	} else {
	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxdecq_pbclass_t;
	    } else {
		linetext_data->context =
		    ((struct lxdecq_type *)
		     ((struct lxtext_type *)
		      linetext_data->context)->text)->rule_f.cxbend + 1;

		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxdecq_p0class_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxdecq_p0class_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxdecq_p0class_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
line_psix(v_case_args_____1)
{
    int *line, type;

    line = *args;

    linetext_data->function.args = line;

    type = nodetype_data->code;

    *line++ = screen(screen)->link_l.type;
    *line++ = type;

    *args = line;

    if (nodetype_data->copy || may_include(lxtype_data)) {
	linetext_data->function.function = _libx1f4i0_lxdecq_pfclass_v;
    } else {
	if (X1f4_E4_LAST < type) {
	    linetext_data->function.function = _libx1f4i0_lxdecq_pfclass_u;
	} else {
	    if (type == X1f4_E4_TEXT) {
		linetext_data->function.function = _libx1f4i0_lxdecq_pfclass_t;
	    } else {
		linetext_data->context =
		    ((struct lxdecq_type *)
		     ((struct lxtext_type *)
		      linetext_data->context)->text)->rule_f.cxbend + 0;

		if (type == X1f4_E4_REAL) {
		    linetext_data->function.function =
			_libx1f4i0_lxdecq_p0class_r;
		} else {
		    if (type == X1f4_E4_MODE) {
			linetext_data->function.function =
			    _libx1f4i0_lxdecq_p0class_m;
		    } else {
			if (type == X1f4_E4_BILL) {
			    linetext_data->function.function =
				_libx1f4i0_lxdecq_p0class_b;
			} else {
			}
		    }
		}
	    }
	}
    }
}


static void
line_qpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_qpix);
}


static void
line_qsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_qsix);
}


static void
line_qsup(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_qsup);
}


static void
line_qsus(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_qsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_qsus);
}


static void
line_sset(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;
    *line++ = X1f4_E4_POST_FLAT | X1f4_E4_POST_XSET;
    *line++ = 0;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
line_xpix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 1, rate_xpix);
}


static void
line_xsix(v_case_args_____1)
{
    _libx1f4i0_lxcall_line_xsix
	(v_case_args_____2, screen(screen)->link_l.type,
	 ((struct lxdecq_type *)
	  ((struct lxtext_type *)
	   linetext_data->context)->text)->rule_f.cxbend + 0, rate_xsix);
}


static void
line_yfix(u_case_args_____1)
{
    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed__l(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


static void
themed_ll(u_case_args_____1)
{
    int *line;

    line = *args;

    linetext_data->function.args = line;

    *line++ = screen(screen)->link_l.type;
    *line++ = screen(screen)->link_l.type;

    *args = line;

    linetext_data->function.type = screen(screen)->link_l.type;
}


int
x1f4_init_lxdecq(void **lxdecq, unsigned bits,
		 const struct screen_type *screen_data)
{
    int (*link)(void *, void **, unsigned), status;
    void *list, *text;

    if (bits & CODELINK_LINK) {
	link = screen_data->lock_w.link;
	text = screen_data->lock_w.data;
    } else {
	link = _x1f4_e4_link_data;
	text = (void *) 0;
    }

    status = link(text, &list, sizeof(struct lxdecq_type));
    if (status) {
	status = LINK_ERROR;
    } else {
	status = init_list(list, bits, screen_data);
	if (status) {
	    if (bits & CODELINK_LINK) {
		screen_data->lock_w.free(text, list);
	    } else {
		_x1f4_e4_free_data(text, list);
	    }
	} else {
	    *lxdecq = list;
	}
    }

    return status;
}
