/*
 * lxline-types.h
 * Copyright (C) 2008-2017, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxline_types_H__
#define __LIBx1f4_lxline_types_H__

#include <config.h>

#include <lxline-config.h>

#include <dt.h>
#include <e4.h>
#include <fd.h>
#include <frame.h>
#include <integral_q.h>
#include <it.h>
#include <ln.h>
#include <lxcast.h>
#include <lxrail.h>
#include <nl.h>
#include <track.h>
#include <trans.h>

#define excase_type			x1f4_nodelink_type
#define exline_type			x1f4_nodeline_type

#define cxline_type			x1f4_leadnode_type

#define here_NULL			((void *) 0)

#define __types_lock(lxline) \
    (&__types_long(lxline)->east_1)

#define __types_long(lxline) \
    (&__types_rail(lxline)->over_1.rule_1.nice_1)

#define __types_rail(lxline) \
    (&((struct lxline_type *) (lxline))->ever_r)

#define here_offsetof(type, ceil) \
    ((integral_q) (&((type *) here_NULL)->ceil))

#define lock_excase(dxline_text) \
    ((struct excase_type *) &lock_cxline((dxline_text))->link)

#define lock_exline(dxline_text) \
    ((struct exline_type *) &lock_cxline((dxline_text))->line)

#define lock_cxline(dxline_text) \
    ((struct cxline_type *)						      \
     ((char *) (dxline_text) - sizeof(struct cxline_type)))

#define near_dxline(excase_text) \
    ((struct dxline_type *)						      \
     ((char *) near_cxline((excase_text)) + sizeof(struct cxline_type)))

#define near_exline(excase_text) \
    ((struct exline_type *) &near_cxline((excase_text))->line)

#define near_cxline(excase_text) \
    ((struct cxline_type *)						      \
     ((char *) (excase_text) - here_offsetof(struct cxline_type, link)))

#define pick_dxline(fxline_text) \
    ((struct dxline_type *)						      \
     ((char *) (fxline_text) + sizeof(struct cxline_type)))

#define post_dxline(dxline_text) \
    ((struct dxline_type *) (dxline_text) + 1)

#define __struct_dxline_type_alloc_size__ \
    sizeof(struct cxline_type) + sizeof(struct dxline_type)

typedef struct cxbend_type {
    int bend, (*gate)(void *, void **, const char **, unsigned *),
	(*pick)(void *, void **), sign, (*slip)(void *);
    void *line;
} cxbend_type;

typedef struct cxseek_type {
    int (*seek)(void *, const char *, void **, const char **, unsigned *);
    void *line;
} cxseek_type;

typedef struct dxline_type {
    unsigned size;
    void *line;
} dxline_type;

typedef struct either_type {
    int roll;
    struct excase_type *excase_data;
    struct lxline_type *lxline_data;
    struct lxtext_type *lxtext_data;
    unsigned tune;
    void *either, *pass;
} either_type;

typedef struct lxline_type {
    struct lxrail_type ever_r;
    struct {
	int data;
    } link_i;
    struct {
	void *data;
    } link_l;
    struct {
	void *fine;
    } link_o;
    struct {
	struct cxbend_type cxbend[2];
	struct cxseek_type cxseek[4];
    } rule_f;
} lxline_type;

typedef struct lxnode_type {
    char *miss;
    struct lxnode_type *last_lxnode, *post_lxnode;
    struct x1f4_dispatch_type dispatch;
    struct x1f4_dxcast_type dxcast;
    unsigned fail;
    void *input[2], *line, *lxline, *transfer;
} lxnode_type;

typedef struct screen_type {
    struct x1f4_track_type lock_e;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } lock_g;
    struct {
	int type;
    } lock_f;
    struct {
	int type;
    } link_l;
    struct x1f4_trans_type lock_m;
    struct {
	int (*call)(void *, struct x1f4_nodelink_type *),
	    (*fine)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *),
	    (*miss)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *);
	void *text;
    } lock_r;
    struct {
	void *fine;
    } link_o;
    struct x1f4_iturn_type lock_p;
    struct x1f4_composer_type lock_q;
    struct {
	unsigned miss;
	const void *data;
    } lock_t;
    struct x1f4_frame_type lock_v;
    struct x1f4_trans_type lock_w;
    struct x1f4_scope_type lock_x;
    struct {
	struct x1f4_linetrap_type *linetrap_data;
    } link_y;
} screen_type;

#endif				/* __LIBx1f4_lxline_types_H__ */
