/*
 * lxlist-a.j.c
 * Copyright (C) 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <e4.h>
#include <lxlist-types.h>
#include <progress.h>

#define LIST_PROGRESS_POSITION \
    X1f4_PROGRESS_EQCLAUSE | X1f4_PROGRESS_GTCLAUSE | X1f4_PROGRESS_LTCLAUSE

static const struct x1f4_position_type position_1x[] = {
/* *INDENT-OFF* */
    {
    {	"l_jack(;1, ;*, ;0)",	1,	1,	0,	1,	18	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"lt_next(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"lt_previous(;1, ;*, ;0)",
				1,	1,	0,	1,	23	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
}, position_2x[] = {
/* *INDENT-OFF* */
    {
    {	"(;0 += ;#) & 0 | lt_regress(;*, ;0, ;1)",
				2,	1,	1,	1,	39	},
	X1f4_PROGRESS_EQCLAUSE
    },
    {
    {	"lt_progress(;*, ;0, ;1)",
				1,	1,	0,	1,	23	},
	X1f4_PROGRESS_GTCLAUSE
    },
    {
    {	"lt_regress(;*, ;0, ;1)",
				1,	1,	0,	1,	22	},
	X1f4_PROGRESS_LTCLAUSE
    },
/* *INDENT-ON* */
};
static const struct x1f4_progress_type progress_1b = {
/* *INDENT-OFF* */
    {	"lt_previous(;1, ;*, ;0)",
				1,	1,	0,	1,	23	},
    {	"lt_last(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
	X1f4_E4_MODE,	X1f4_E4_SLIP,	0,
	X1f4_PROGRESS_BACKWARD | LIST_PROGRESS_POSITION,	2,
	position_1x
/* *INDENT-ON* */
}, progress_1f = {
/* *INDENT-OFF* */
    {	"lt_next(;1, ;*, ;0)",	1,	1,	0,	1,	19	},
    {	"lt_first(;1, ;*, ;0)",	1,	1,	0,	1,	20	},
	X1f4_E4_MODE,	X1f4_E4_SLIP,	0,
	LIST_PROGRESS_POSITION,		2,
	position_1x
/* *INDENT-ON* */
}, progress_2b = {
/* *INDENT-OFF* */
    {	"lt_regress(;*, ;0, ;1)",
				1,	1,	0,	1,	22	},
    {	"(;0 = 0) | lt_regress(;*, ;0, ;1)",
				2,	1,	0,	1,	33	},
	X1f4_E4_MODE,	X1f4_E4_SIDE,	0,
	LIST_PROGRESS_POSITION | X1f4_PROGRESS_BACKWARD,	2,
	position_2x
/* *INDENT-ON* */
}, progress_2f = {
/* *INDENT-OFF* */
    {	"lt_progress(;*, ;0, ;1)",
				1,	1,	0,	1,	23	},
    {	"(;0 = -;#) && lt_progress(;*, ;0, ;1)",
				2,	1,	1,	1,	37	},
	X1f4_E4_MODE,	X1f4_E4_SIDE,	0,
	LIST_PROGRESS_POSITION,		2,
	position_2x
/* *INDENT-ON* */
};

int
x1f4_pass_lxlist(void *lxlist, struct x1f4_progress_type **progress)
{
    int type;
    struct x1f4_progress_type *progress_data;

    type = ((struct lxlist_type *) lxlist)->link_t.datatype[0].type;

    progress_data = *progress;

    *progress_data = progress_1f;
    progress_data->type = type;
    progress_data++;

    *progress_data = progress_1b;
    progress_data->type = type;
    progress_data++;

    *progress_data = progress_2f;
    progress_data->type = type;
    progress_data++;

    *progress_data = progress_2b;
    progress_data->type = type;
    progress_data++;

    *progress = progress_data;

    return 0;
}

