/*
 * lxlist-c.c.c
 * Copyright (C) 2013-2015, 2017-2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>

int
_libx1f4i0_lxlist_c_centric(void *context, void *output, void **input)
{
    X1f4_E4_C_MODE mode;
    int status;
    struct dxlist_type *dxlist_data;
    struct lxlist_type *lxlist_data;
    unsigned size;

    lxlist_data = context;

    dxlist_data = I_LIST(input[0]);

    mode = I_MODE(input[1]);

    size = dxlist_data->size + 1;
    if (mode < -(X1f4_E4_C_MODE) (int) size
	|| !(mode < (X1f4_E4_C_MODE) (int) size)) {
	status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);
    } else {
	struct x1f4_function_type *function_data;
	unsigned count;
	void **strip;

	if (mode < 0) {
	    mode += size;
	}

	strip = input[-1];

	function_data = strip[X1f4_E4_RENDITION];

	status = 0;

	count = function_data->count - 2;
	if (count) {
	    struct x1f4_function_type function;
	    void *slip[4];

	    slip[0] = strip;
	    slip[1] = *input;
	    slip[2] = &mode;

	    input++;

	    /*
	     * pretty error reporting:
	     */
	    function.length = 9;
	    function.name = "l_collect";
	    function.type = X1f4_E4_VOID;

	    function.args = function_data->args;

	    strip[X1f4_E4_RENDITION] = &function;

	    for (; count; count--) {
		slip[3] = *++input;
		status =
		    _libx1f4i0_lxlist_l_pending(lxlist_data, output, slip + 1);
		if (status) {
		    break;
		} else {
		    mode++;
		}

		function.args++;
	    }
	}

	if (status) {
	} else {
	    l_LIST(dxlist_data, output);
	}
    }

    return status;
}
