/*
 * lxlist-m.r.c
 * Copyright (C) 2008-2011, 2013, 2016-2018, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxlist-config.h>

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_r_class_m(void *context, void *output, void **input)
{
    int status;
    struct dxlist_type *dxlist_data;

    dxlist_data = I_LIST(input[0]);

    do {
	X1f4_E4_C_MODE mode;
	struct lxlist_type *lxlist_data;
	struct lxtext_type *lxtext_data;
	unsigned size;
	void *data;

	lxtext_data = context;

	lxlist_data = lxtext_data->text;

	mode = I_MODE(input[1]);

	size = dxlist_data->size;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);

	    break;
	}

	if (mode < 0) {
	    mode += size;
	}

	status = _libx1f4i0_peek_l4list(dxlist_data->list, mode, &data);
	if (status) {
	    status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);
	} else {
	    struct lxtext_type *lxtext_link;
	    unsigned link;

	    lxtext_link = lxlist_data->link_f.text;

	    link = effect_link(data);

	    if (~link) {
		int (*free)(void *, void *, struct excase_type **,
			    const struct excase_type *);
		struct lxtext_type *lxtext_slip;

		lxtext_slip = lxtext_link + link;

		free = lxtext_slip->lxtype.free;
		if (free) {
		    status = free
			(lxtext_slip->lxtype.context, E_USER(data), NULL,
			 lock_excase(dxlist_data));
		}
	    }
	    {
#define type				X1f4_E4_MODE

		X1f4_E4_C_MODE trip;

		type_effect(data, type);

		link_effect(data, ~0);

		trip = I_MODE(input[2]);

		l_MODE(trip, effect_data(data));
		l_MODE(trip, output);

#undef type
	    }
	}
    } while (0);

    return status;
}

