/*
 * lxlist-n.p.c
 * Copyright (C) 2008-2011, 2013-2017, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_p_nearest(void *context, void *output, void **input)
{
    int status;

    do {
	X1f4_E4_C_MODE mode, rule;
	struct cxnear_type *cxnear_data;
	struct dxlist_type *dxlist_data;
	struct lxlist_type *lxlist_data;
	unsigned size;
	void *list, *side;

	cxnear_data = context;

	mode = I_MODE(input[1]);

	lxlist_data = cxnear_data->list;

	dxlist_data = I_LIST(input[0]);

	size = dxlist_data->size + 1;
	if (mode < -(X1f4_E4_C_MODE) (int) size
	    || !(mode < (X1f4_E4_C_MODE) (int) size)) {
	    status = _libx1f4i0_lxlist_stat_deck(lxlist_data, size, mode);

	    break;
	} else {
	    if (mode < 0) {
		mode += size;
	    }
	}

	size--;

	rule = I_MODE(input[2]);

	if (rule < 1) {
	    if (rule < 0) {
		status = _libx1f4i0_lxlist_stat_debt(lxlist_data, rule);

		break;
	    } else {
		status = 0;
	    }

	    break;
	}

	list = dxlist_data->list;

	/*
	 * switch value determined empirically
	 */
	if (rule < 384 && 31 < size) {
	    int hint = 0, (*roll)(void *, void *);
	    void *fill;

	    roll = cxnear_data->rule;
	    fill = input[3];

	    while (rule) {
		unsigned into;
		void *post;

		into = rule;
		status = _libx1f4i0_vamp_l4list
		    (list, mode, &hint, &into, &post);
		if (status) {
		    break;
		} else {
		    rule -= into;
		    size += into;

#if 1
		    mode += into * (hint ^ 1);
		    hint = 0;

		    for (; into; into--) {
			/* setting up an item outside the loop and memcpy it
			   here is not faster */
			roll(fill, post);
			post = (char *) post + effect_size;
		    }
#else
		    status = _libx1f4i0_over_l4list
			(list, mode, mode + into - 1, input[3],
			 cxnear_data->rule);
		    if (status) {
			/*
			 * assuming this one cannot fail.  othewise curing
			 * the list is required.
			 */
		    } else {
			mode += into * (hint ^ 1);
		    }
#endif				/* 1 */
		}
	    }

	    dxlist_data->size = size;
	} else {
	    status = _libx1f4i0_lxlist_long_list(lxlist_data, &side);
	    if (status) {
	    } else {
		int excess;

		if (mode == size || !mode) {
		    status = _libx1f4i0_star_l4list(side, rule);
		    if (status) {
		    } else {
			status = _libx1f4i0_lime_l4list
			    (side, input[3], cxnear_data->rule);
			if (status) {
			} else {
			    if (mode) {
				status = _libx1f4i0_join_l4list(list, side);
			    } else {
				status = _libx1f4i0_join_l4list(side, list);
				if (status) {
				} else {
				    status =
					_libx1f4i0_swap_l4list(list, side);
				}
			    }
			    if (status) {
			    } else {
				dxlist_data->size = size + rule;
			    }
			}
		    }

		    if (status) {
			status = _libx1f4i0_lxlist_stat_list
			    (lxlist_data, 0, status);
		    }
		} else {
		    void *trip;

		    status = _libx1f4i0_lxlist_long_list(lxlist_data, &trip);
		    if (status) {
		    } else {
			status = _libx1f4i0_star_l4list(side, rule);
			if (status) {
			} else {
			    status = _libx1f4i0_tear_l4list(list, mode, trip);
			    if (status) {
			    } else {
				status = _libx1f4i0_lime_l4list
				    (side, input[3], cxnear_data->rule);
				if (status) {
				} else {
				    status =
					_libx1f4i0_join_l4list(list, side);
				    if (status) {
				    } else {
					status = _libx1f4i0_join_l4list
					    (list, trip);
					if (status) {
					} else {
					    dxlist_data->size = size + rule;
					}
				    }
				}

				if (status) {
				    dxlist_data->list = trip;
				    _libx1f4i0_size_l4list
					(trip, &dxlist_data->size);

				    _libx1f4i0_lxlist_null_list
					(lxlist_data, NULL, dxlist_data);

				    dxlist_data->list = list;
				    _libx1f4i0_size_l4list
					(list, &dxlist_data->size);
				}
			    }
			}

			excess = _libx1f4i0_fini_l4list(&trip);
			if (excess) {
			    if (status) {
			    } else {
				status = excess;
			    }
			}

			if (status) {
			    status = _libx1f4i0_lxlist_stat_list
				(lxlist_data, 0, status);
			}
		    }
		}

		excess = _libx1f4i0_fini_l4list(&side);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}
	    }
	}
    } while (0);

    return status;
}
