/*
 * lxlist-v.p.b.c
 * Copyright (C) 2008-2011, 2013, 2014, 2017-2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <lxlist-config.h>

#include <stddef.h>

#include <e4.h>
#include <l_list.h>
#include <lxcall.h>
#include <lxlist-drive.h>
#include <lxlist-inter.h>
#include <lxlist-types.h>

int
_libx1f4i0_lxlist_pbclass_v(void *context, void *output, void **input)
{
    int status;

    do {
	struct dxlist_type *dxlist_data;
	struct lxlist_type *lxlist_data;
	struct lxtext_type *lxtext_data;
	void *slip;

	lxtext_data = context;

	lxlist_data = lxtext_data->text;

	dxlist_data = I_LIST(input[0]);

	status = lxtext_data->lxtype.copy
	    (lxtext_data->lxtype.context, NULL, &slip,
	     I_USER(input[1]), lock_excase(dxlist_data));
	if (status) {
	    break;
	} else {
	    unsigned type;
	    void *data;

	    type = lxtext_data->lxtype.code;

#if defined __MISSING_TAIL_L_LIST__
	    status = _libx1f4i0_push_l4list
		(dxlist_data->list, dxlist_data->size, &data);
#else
	    status = _libx1f4i0_tail_l4list(dxlist_data->list, &data);
#endif				/* __MISSING_TAIL_L_LIST__ */
	    if (status) {
		status = _libx1f4i0_lxlist_stat_list(lxlist_data, 0, status);

		status = lxtext_data->lxtype.free
		    (lxtext_data->lxtype.context,
		     slip, NULL, lock_excase(dxlist_data));
	    } else {
		unsigned link;

		type_effect(data, type);

		link = lxtext_data
		    - ((struct lxtext_type *) lxlist_data->link_f.text);

		link_effect(data, link);

		dxlist_data->size++;

		l_USER(slip, effect_data(data));

		if (lxtext_data->lxtype.lead) {
		    status = x1f4_auto_node
			(&lxlist_data->link_v, &lxlist_data->link_e,
			 &lxtext_data->lxtype, &slip, output);
		} else {
		    l_USER(I_USER(input[1]), output);
		}
	    }
	}
    } while (0);

    return status;
}

