/*
 * lxpart-types.h
 * Copyright (C) 2008-2010, 2013-2015, 2017, 2019, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxpart_types_H__
#define __LIBx1f4_lxpart_types_H__

#include <time.h>

#include <dt.h>
#include <frame.h>
#include <integral_q.h>
#include <ln.h>
#include <lxmile.h>
#include <track.h>
#include <trans.h>
#include <nt.h>

#define lxtext_type			lxfold_type

#define here_NULL                       ((void *) 0)

#define here_offsetof(type, ceil) \
    ((integral_q) (&((type *) here_NULL)->ceil))

#define lock_excase(dxpart_text) \
    ((struct x1f4_nodelink_type *) &lock_cxpart((dxpart_text))->link)

#define lock_expart(dxpart_text) \
    ((struct x1f4_nodeline_type *) &lock_cxpart((dxpart_text))->line)

#define lock_cxpart(dxpart_text) \
    ((struct x1f4_leadnode_type *)                                            \
     ((char *) (dxpart_text) - sizeof(struct x1f4_leadnode_type)))

#define near_dxpart(excase_text) \
    ((struct dxpart_type *)						      \
     ((char *) near_fxpart((excase_text)) + sizeof(struct x1f4_leadnode_type)))
#define near_expart(excase_text) \
    ((struct x1f4_nodeline_type *) &near_fxpart((excase_text))->line)

#define near_fxpart(excase_text) \
    ((struct x1f4_leadnode_type *)					      \
     ((char *) (excase_text) - here_offsetof(struct x1f4_leadnode_type, link)))

#define pick_dxpart(fxpart_text) \
    ((struct dxpart_type *)						      \
     ((char *) (fxpart_text) + sizeof(struct x1f4_leadnode_type)))

#define __struct_dxpart_type_alloc_size__ \
    sizeof(struct x1f4_leadnode_type) + 0

typedef struct cxpart_type {
    const char *name;
    int (*copy)(void *, void *, void *, const void *,
		struct x1f4_caselink_type **), (*flat)(void *, void *),
	(*high)(void *, void *, struct x1f4_nodelink_type **),
	(*line)(void *, void *), type;
    unsigned size, trap;
    void *context;
} cxpart_type;

typedef struct dxpart_type {
    void *none;
} dxpart_type;

typedef struct lxpart_type {
    struct lxmile_type ever_n;
    struct {
	void *part;
    } line_f;
    struct {
	unsigned miss;
    } link_l;
    struct {
	struct x1f4_datatype_type *datatype_data;
    } link_t;
} lxpart_type;

typedef struct screen_type {
    struct x1f4_track_type lock_e;
    struct {
	struct cxpart_type *cxpart_data;
	unsigned miss;
    } link_l;
    struct x1f4_trans_type lock_m;
    struct {
	int (*call)(void *, struct x1f4_nodelink_type *),
	    (*fine)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *),
	    (*miss)(void *, struct x1f4_nodelink_type **,
		    struct x1f4_nodelink_type *);
	void *text;
    } lock_r;
    struct x1f4_frame_type lock_v;
    struct x1f4_trans_type lock_w;
} screen_type;

#endif				/* __LIBx1f4_lxpart_types_H__ */
