/*
 * lxrail-s.g.c
 * Copyright (C) 2011-2018, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxrail-drive.h>
#include <lxrail-inter.h>
#include <lxrail-types.h>

#define lxrail(dash) \
    ((struct lxrail_type *) (dash))

typedef struct pick_type {
    struct fxdata_type finite[2];
    struct x1f4_dxcast_type dxcast, serial[2];
    struct x1f4_function_type function;
    struct x1f4_linetext_type linetext;
} pick_type;

/*
 * function does setup for an unused output
 */
int
_libx1f4i0_lxrail_form_call(void *lxrail, void *output, void **input,
			    struct x1f4_linetext_type *linetext_data,
			    unsigned count,
			    const struct x1f4_function_type *function_text,
			    int lead, int jack, int lock, int over,
			    void *affect)
{
    int linked, status;
    unsigned class, grade, parts, scale = 0;
    void *beta;

    class = function_text->count - lead;

    grade = class + 2;

    parts = function_text->flags;

    if (parts & X1f4_E4_POST_TYPE) {
	if (linetext_data->function.flags
	    & (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
	    scale = 1;
	} else {
	    parts &= ~X1f4_E4_POST_TYPE;
	}
    }

    linked = sizeof(struct pick_type);

    status = lxrail(lxrail)->link_v.link
	(lxrail(lxrail)->link_v.data, &beta,
	 linked + (sizeof(void *) + (sizeof(int) << scale)) * grade
	 + sizeof(void *));
    if (status) {
	status = _libx1f4i0_lxrail_stat_link(lxrail);
    } else {
	int *args, deck[2], *reach, *still, type;
	struct affect_type *affect_data;
	struct center_type center;
	struct covert_type covert;
	struct pick_type *call_data;
	struct x1f4_function_type *function_call;
	void **frame, **order, **plane;

	call_data = beta;

	function_call = &call_data->function;

	function_call->count = grade;

	function_call->flags = parts & X1f4_E4_POST_TYPE;

	frame = (void *) (call_data + 1);
	frame++;

	function_call->args = args = (void *) (frame + grade);

	frame[-1] = input[-1];

	type = linetext_data->function.type;

	if (type != X1f4_E4_CASE) {
	    l_USER(&call_data->dxcast, output);

	    call_data->dxcast.call = 0;
	    call_data->dxcast.type = type;

	    output = &call_data->dxcast.data;
	} else {
	}

	_libx1f4i0_lxcall_keep_call
	    (frame, input, parts, args, function_text->args, class, lead, jack,
	     lock);

	if (jack < lock) {
	} else {
	    jack++;
	}

	args[jack] = over;

	deck[0] = jack;
	deck[1] = lock;

	reach = args + jack;
	still = args + lock;

	order = frame + jack;
	plane = frame + lock;

	affect_data = affect;

	*still = affect_data->type;
	*order = &affect_data->fxtext;
	*plane = &affect_data->fxdata;

	center.composer_data = &lxrail(lxrail)->link_q;
	center.eelookup_data = lxrail(lxrail)->trap_e.eelookup_data;
	center.function_text = &linetext_data->function;
	center.iturn_data = &lxrail(lxrail)->link_p;
	center.track_data = &lxrail(lxrail)->link_e;
	center.input = &frame;

	covert.frame_data = &lxrail(lxrail)->link_v;
	covert.scope_data = &lxrail(lxrail)->link_x;
	covert.swap = NULL;

	{
	    /* the args, count, flags set should do */
	    call_data->linetext = *linetext_data;

	    call_data->linetext.function.type = X1f4_E4_CASE;

	    status = _libx1f4i0_lxrail_long_call
		(&call_data->linetext, function_call, output, frame, &frame,
		 &center, &covert, lxrail, linetext_data);
	    if (status) {
		count = 0;
	    } else {
		count--;
	    }
	}
	for (; count; count--) {
	    affect_data++;

	    *still = affect_data->type;
	    *order = &affect_data->fxtext;
	    *reach = over;
	    *plane = &affect_data->fxdata;

	    status = _libx1f4i0_lxcall_rule_call
		(&call_data->linetext, function_call, output, &center,
		 &covert, 2, deck, call_data->finite, call_data->serial);
	    if (status) {
		break;
	    }
	}
	{
	    int excess;

	    excess = _libx1f4i0_lxcall_near_call(&covert);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	}
    }

    return status;
}
