/*
 * lxroll-a.a.c
 * Copyright (C) 2008-2014, 2016, 2017, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <unistd.h>

#include <sys/types.h>

#include <e4.h>
#include <lxroll-defs.h>
#include <variable.h>

#define READ_BUFFER			0

typedef struct constant_type {
    struct x1f4_variable_type variable;
    const void *data;
} constant_type;

static X1f4_E4_C_MODE names[] = {
/* *INDENT-OFF* */
    NONBLOCKING_INPUT,
    NONBLOCKING_OUTPUT,
    REUSE_ADDRESS,
    TCP_NO_DELAY
/* *INDENT-ON* */
};
static const struct constant_type constants[] = {
/* *INDENT-OFF* */
    {	{	"NONBLOCKING_INPUT",	X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	17			},
	names + 0							},
    {	{	"NONBLOCKING_OUTPUT",	X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	18			},
	names + 1							},
    {	{	"REUSE_ADDRESS",	X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	13			},
	names + 2							},
    {	{	"TCP_NODELAY",		X1f4_E4_MODE,
		X1f4_E4_READ_ONLY,	11			},
	names + 3							}
/* *INDENT-ON* */
};

int
x1f4_vset_lxroll(void *lxroll, void **context)
{
    int status;
    const struct constant_type *constant_data;
    unsigned i;

    constant_data = constants;

    i = sizeof(constants) / sizeof(struct constant_type);
    for (; i; i--) {
	status = x1f4_push_variable
	    (context, constant_data->variable.name,
	     constant_data->variable.length, constant_data->variable.type,
	     constant_data->variable.flags, (void *) constant_data->data);
	if (status) {
	    break;
	} else {
	    constant_data++;
	}
    }

    return status;
}
