/*
 * lxroll-d.l.c
 * Copyright (C) 2017, 2019, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <unistd.h>

#include <netinet/tcp.h>

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>

#include <e4.h>
#include <lxfile.h>
#include <lxroll-defs.h>
#include <lxroll-drive.h>
#include <lxroll-inter.h>
#include <lxroll-types.h>

static int case_unix(void *, const char *, unsigned, int);

static int
case_unix(void *lxroll, const char *path, unsigned bits, int server)
{
    int status;

    do {
	struct sockaddr_un address;


	memset(&address, 0, sizeof(struct sockaddr_un));
	address.sun_family = AF_UNIX;
	strcpy(address.sun_path, path);
	status = bind(server, (struct sockaddr *) &address, sizeof(address));
	if (status) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll, "bind");
	} else {
	    status = listen(server, 1);
	    if (status) {
		status = _libx1f4i0_lxroll_stat_call(lxroll, "listen");
	    }
	}
    } while (0);

    return status;
}


int
_libx1f4i0_lxroll_l_derived(void *context, void *output, void **input)
{
    int fd = -1, status;

    do {
	const char *path;
        struct lxroll_type *lxroll_data;
        struct x1f4_dxfile_type *dxfile_data;
        void *lxfile;

        lxroll_data = context;

	path = I_TEXT(input[1]);

	if (strlen(path) < sizeof(((struct sockaddr_un *) 0)->sun_path)) {
	} else {
	    status = _libx1f4i0_lxroll_stat_fill(lxroll_data);
	    if (1) {
		break;
	    }
	}

        lxfile = lxroll_data->link_l.file;

        dxfile_data = I_FILE(input[0]);

	fd = socket(AF_UNIX, SOCK_STREAM, 0);
	if (fd == -1) {
	    status = _libx1f4i0_lxroll_stat_call(lxroll_data, "socket");
	    if (1) {
		break;
	    }
	}

	status = case_unix(lxroll_data, path, I_MODE(input[2]), fd);
	if (status) {
	    break;
	}

	if (dxfile_data->file) {
	    status = x1f4_lose_dxfile(lxfile, dxfile_data);
	    if (status) {
		break;
	    }
	}

	status = x1f4_pick_dxfile(lxfile, dxfile_data, fd, 0);
	if (status) {
	    break;
	}

	l_FILE(dxfile_data, output);

	fd = -1;
    } while (0);

    if (fd ^ -1) {
	close(fd);
    }

    return status;
}
