/*
 * lxtrap-a.m.c
 * Copyright (C) 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <lxcall.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

int
_libx1f4i0_lxtrap_suit_line(struct lxtrap_type *lxtrap_data,
			    const struct x1f4_function_type *function_text,
			    const struct x1f4_function_type *function_data,
			    unsigned offset, struct covert_type *covert_data,
			    void **track)
{
    const int *args, *line;
    int status = 0;
    unsigned count, flags, grade, i;

    flags = function_text->flags;

    covert_data->text = lxtrap_data;

    grade = function_data->count;
    line = function_data->args + grade + offset;

    count = function_text->count;
    args = function_text->args + count;

    count += offset;

    for (i = offset; i < grade; i++, line++) {
	int lead;

	if (i < count) {
	    lead = *args++ & *line & X1f4_E4_POST_XSET;
	} else {
	    if (flags & X1f4_E4_LINK_PASS) {
		lead = *line & X1f4_E4_POST_XSET;
	    } else {
		break;
	    }
	}
	if (lead) {
	    int type;
	    void *data;

	    data = track[i];

	    /*
	     * can't use call function arg types, they may not match
	     * the passed args.  use the latter.
	     */
	    type = *(line - grade);
	    if (type == X1f4_E4_TEXT) {
		status = _libx1f4i0_lxcall_line_text
		    (&lxtrap_data->link_v, &lxtrap_data->link_m,
		     &lxtrap_data->link_e, data);
	    } else {
		status =
		    _libx1f4i0_lxtrap_suit_node(NULL, covert_data, data, type);
	    }
	    if (status) {
		break;
	    }
	}
    }

    return status;
}
