/*
 * lxtrap-i.v.c
 * Copyright (C) 2011-2015, 2017, 2019, 2022, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-drive.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>

typedef struct fxmiss_type {
    struct {
	int land, type;
	void *report;
    } lead;
    void *fxcase, *locker, *lxtrap, **track, *zeta;
} fxmiss_type;

extern const char *const x1f4_c1_empty_string;
extern const struct x1f4_track_type _libx1f4i0_missed_track,
    _libx1f4i0_stored_track;

extern const void *const x1f4_a1_walk_link;

static int miss_call(void *, int, void *, struct x1f4_dispatch_type **);
static int slip_call(void *);

static int
miss_call(void *text, int status, void *output,
	  struct x1f4_dispatch_type **dispatch)
{
    struct fxmiss_type *fxmiss_data;

    fxmiss_data = text;

    fxmiss_data->track[-1] = fxmiss_data->zeta;

    if (fxmiss_data->locker) {
	struct lxtrap_type *lxtrap_data;
	struct x1f4_track_type **trace, *track_data;

	lxtrap_data = fxmiss_data->lxtrap;

	trace = lxtrap_data->link_h.call;

	track_data = *trace;

	*trace = fxmiss_data->locker;

	if (status == X1f4_EX_CAN_CONTINUE) {
	    if (track_data != &_libx1f4i0_missed_track) {
		struct fxcase_type *fxcase_data;

		fxcase_data = fxmiss_data->fxcase;
		if (fxcase_data->missed) {
		} else {
		    int excess;
		    void *record;

		    record = fxcase_data->record;

		    excess = _libx1f4i0_lxcall_pick_text
			(&lxtrap_data->ever_a, record);
		    if (excess) {
			status = excess;
		    } else {
			l_TEXT(fxcase_data->text, record);

			fxcase_data->text = (char *) x1f4_c1_empty_string;
		    }
		}
	    }
	}
    }

    if (status) {
	if (status == X1f4_EX_CAN_CONTINUE) {
	    status = 0;
	    l_MODE(-1, output);
	}
    } else {
	if (fxmiss_data->lead.report) {
	    status = _libx1f4i0_lxtrap_lock_port
		(fxmiss_data->lead.land, fxmiss_data->lead.report,
		 fxmiss_data + 1, fxmiss_data->lead.type, fxmiss_data->lxtrap);
	    if (status) {
	    } else {
		l_MODE(status, output);
	    }
	} else {
	    l_MODE(0, output);
	}
    }

    *dispatch = NULL;

    return status;
}


static int
slip_call(void *call)
{
    int status;
    struct fxcase_type *fxcase_data;
    void *text;

    fxcase_data = call;

    text = fxcase_data->text;
    if (text != x1f4_c1_empty_string) {
	status = fxcase_data->trans.free(fxcase_data->trans.data, text);
	if (status) {
	    status = X1f4_EX_CRITICAL;
	}
    } else {
	status = 0;
    }

    return status;
}


int
_libx1f4i0_lxtrap_slip_call(void *output, int land, void *report, int offset,
			    void **input, void *meta, void **track, void *side,
			    struct lxtrap_type *lxtrap_data,
			    struct x1f4_linetext_type *linetext_data,
			    int frame)
{
    int status;

    do {
	unsigned clip, mind;
	void *miss, **port = NULL;

	if (frame == 2) {
	    clip = sizeof(struct fxcase_type) + sizeof(struct x1f4_track_type);
	    offset++;
	} else {
	    clip = 0;
	}

	if (linetext_data->function.flags
	    & (X1f4_E4_SIDE_LIST | X1f4_E4_SLIP_LIST)) {
	    mind = sizeof(struct x1f4_linetext_type);
	    if (linetext_data->function.flags
		& (X1f4_E4_LINK_PASS | X1f4_E4_POST_TYPE)) {
		mind +=
		    (((struct x1f4_function_type *) side)->count - offset)
		    * sizeof(int) << 1;
	    } else {
		if (linetext_data->function.count) {
		    mind += sizeof(int)
			* (((struct x1f4_function_type *) side)->count
			   - offset);
		}
	    }
	} else {
	    mind = 0;
	}

	if (frame ^ 2) {
	    status = lxtrap_data->link_v.link
		(lxtrap_data->link_v.data, &miss,
		 sizeof(struct x1f4_dispatch_type) + sizeof(struct fxmiss_type)
		 + sizeof(struct fxdata_type) + clip + mind);
	} else {
	    status = lxtrap_data->link_v.pick
		(lxtrap_data->link_v.data, &port, slip_call,
		 sizeof(struct x1f4_dispatch_type) + sizeof(struct fxmiss_type)
		 + sizeof(struct fxdata_type) + clip + mind, &miss);
	}
	if (status) {
	    status = _libx1f4i0_lxtrap_stat_link(lxtrap_data);
	    break;
	} else {
	    struct fxdata_type *fxdata_data;
	    struct fxmiss_type *fxmiss_data;
	    struct x1f4_dispatch_type *dispatch_data;
	    void *subtext;

	    dispatch_data = miss;

	    fxmiss_data = (void *) (dispatch_data + 1);

	    fxdata_data = (void *) (fxmiss_data + 1);

	    output = input[0];

	    subtext = (void *) linetext_data->context;

	    fxmiss_data->track = track;
	    fxmiss_data->zeta = track[-1];

	    /*
	     * before the RENDITION setting _slip_call 
	     */
	    track[-1] = meta;

	    if (mind) {
		status = _libx1f4i0_lxcall_slip_call
		    (track, side, offset, 0, &linetext_data,
		     (void *) ((char *) (fxdata_data + 1) + clip), &subtext);
		if (status) {
		    if (status == X1f4_EX_CAN_CONTINUE) {
			status = 0;
			input[2] = NULL;
			l_MODE(-1, output);
		    } else {
		    }

		    break;
		}
	    } else {
		((void **) meta)[X1f4_E4_RENDITION] = &linetext_data->function;
	    }

	    fxmiss_data->lead.type = linetext_data->function.type;
	    fxmiss_data->lead.land = land;
	    fxmiss_data->lead.report = report;

	    fxmiss_data->lxtrap = lxtrap_data;

	    dispatch_data->back = fxmiss_data;
	    dispatch_data->call = miss_call;

	    dispatch_data->transfer.context = subtext;
	    dispatch_data->transfer.function_data = &linetext_data->function;
	    dispatch_data->transfer.input = track;
	    dispatch_data->transfer.output = fxdata_data;

	    input[2] = dispatch_data;

	    if (frame) {
		fxmiss_data->locker = *lxtrap_data->link_h.call;

		if (frame == 1) {
		    fxmiss_data->fxcase = NULL;

		    *lxtrap_data->link_h.call =
			(struct x1f4_track_type *) &_libx1f4i0_missed_track;
		} else {
		    struct fxcase_type *fxcase_data;
		    struct x1f4_track_type *track_data;

		    track_data = (void *) (fxdata_data + 1);

		    fxcase_data = (void *) (track_data + 1);

		    *track_data = _libx1f4i0_stored_track;

		    track_data->data = fxcase_data;

		    *lxtrap_data->link_h.call = track_data;

		    fxcase_data->missed = 1;
		    fxcase_data->record = *(void **) input[1];
		    fxcase_data->text = (X1f4_E4_C_TEXT) x1f4_c1_empty_string;
		    fxcase_data->trans = lxtrap_data->link_m;

		    fxmiss_data->fxcase = fxcase_data;

		    *port = fxcase_data;
		}
	    } else {
		fxmiss_data->locker = NULL;
	    }
	}
    } while (0);

    return status;
}
