/*
 * sfar.e.c
 * Copyright (C) 2009-2012, 2016, 2021, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <sfas-defs.h>
#include <sfas-field.h>
#include <sfas-inter.h>
#include <sfas-names.h>
#include <sfas-patch.h>
#include <sfas-types.h>

#define true(e)				1

#define back(miss)			((const char *) (miss))
#define fast(miss)			((const byte *) (miss))

#define sfnote(sfnote)			((struct sfnote_type *) (sfnote))

static int copy_pass(struct sfnote_type *, struct fpnode_type *,
		     struct fpnode_type *, struct fpnode_type *, int,
		     struct fpnode_type *);
static int copy_text(struct sfnote_type *, struct fpnode_type **,
		     struct fpnode_type **, struct fpnode_type **,
		     struct fpnode_type *);
static int join_node(struct sfnote_type *, struct fpnode_type *,
		     struct fpnode_type *);
static int pick_node(struct sfnote_type *, struct fpnode_type *,
		     struct fpnode_type *);
static int tear_note(struct sfnote_type *, const char *, int,
		     struct sfnote_type *);
static int wrap_ever(struct sfnote_type *);
static int wrap_lead(struct sfnote_type *);

static int
copy_pass(struct sfnote_type *sfnote_text, struct fpnode_type *fpnode_data,
	  struct fpnode_type *fpnode_text, struct fpnode_type *fpnode_slip,
	  int slip, struct fpnode_type *fpnode_wrap)
{
    int status;
    struct fpnode_type *fpnode_lead;
    unsigned bits, chance, change, spread;

    bits = fpnode_wrap->bits;

    spread = Spread(bits);

    fpnode_lead = fpnode_wrap->data;

    change = fpnode_slip + slip - fpnode_lead;
    chance = spread - change;
    if (chance & ~1 && change & ~1) {
	void *node;

	status = sfnote_text->link_m.link
	    (sfnote_text->link_m.data, &node,
	     sizeof(struct fpnode_type) * ((chance + 3) & ~3));
	if (status) {
	    status = LINK_ERROR;
	} else {
	    fpnode_text->bits = kset(fpnode_text->bits, chance);
	    fpnode_text->data = node;

	    memcpy(node, fpnode_lead + change,
		   sizeof(struct fpnode_type) * chance);

	    fpnode_data->bits = iset(fpnode_data->bits, change);
	    fpnode_data->data = fpnode_lead;

	    if (((spread + 3) & ~3) != ((change + 3) & ~3)) {
		sfnote_text->link_m.mode
		    (sfnote_text->link_m.data, &fpnode_data->data,
		     sizeof(struct fpnode_type) * ((change + 3) & ~3));
	    }
	}
    } else {
	status = 0;

	if (change & ~1) {
	    fpnode_data->bits = iset(fpnode_data->bits, change);
	    fpnode_data->data = fpnode_lead;

	    if (chance) {
		status = pick_node
		    (sfnote_text, fpnode_text, fpnode_lead + change);
	    } else {
		fpnode_text->bits = kset(fpnode_text->bits, 1);
		fpnode_text->data = NULL;
	    }

	    if (!(change & 3)) {
		sfnote_text->link_m.mode
		    (sfnote_text->link_m.data, &fpnode_data->data,
		     sizeof(struct fpnode_type) * change);
	    }
	} else {
	    if (chance & ~1) {
		if (change) {
		    status = pick_node
			(sfnote_text, fpnode_data, fpnode_lead);

		    memmove(fpnode_lead, fpnode_lead + 1,
			    sizeof(struct fpnode_type) * chance);
		} else {
		    fpnode_data->bits = kset(fpnode_data->bits, 1);
		    fpnode_data->data = NULL;
		}

		fpnode_text->bits = kset(fpnode_text->bits, chance);
		fpnode_text->data = fpnode_lead;

		if (!(chance & 3)) {
		    sfnote_text->link_m.mode
			(sfnote_text->link_m.data, &fpnode_text->data,
			 sizeof(struct fpnode_type) * chance);
		}

	    } else {
		int excess;

		status = pick_node
		    (sfnote_text, fpnode_data, fpnode_lead + 0);

		excess = pick_node
		    (sfnote_text, fpnode_text, fpnode_lead + 1);
		if (excess) {
		    if (status) {
		    } else {
			status = excess;
		    }
		}

		excess = sfnote_text->link_m.free
		    (sfnote_text->link_m.data, fpnode_lead);
		if (excess) {
		    if (status) {
		    } else {
			status = FREE_ERROR;
		    }
		}
	    }
	}
    }

    return status;
}


static int
copy_text(struct sfnote_type *sfnote_text, struct fpnode_type **fpdata,
	  struct fpnode_type **fptext, struct fpnode_type **fpslip,
	  struct fpnode_type *fpnode_wrap)
{
    int status;

    if (fpnode_wrap) {
	struct fpnode_type *fpnode_data, *fpnode_lead, *fpnode_slip;
	unsigned bits, chance, change, spread;

	bits = fpnode_wrap->bits;

	spread = Spread(bits);

	fpnode_slip = *fpslip;

	fpnode_lead = fpnode_wrap->data;

	change = fpnode_slip - fpnode_lead;
	chance = spread - change;
	change += 1;
	if (change & ~1 && chance & ~1) {
	    void *node;

	    status = sfnote_text->link_m.link
		(sfnote_text->link_m.data, &node,
		 sizeof(struct fpnode_type) * ((chance + 3) & ~3));
	    if (status) {
		status = LINK_ERROR;
	    } else {
		unsigned length;

		fpnode_data = *fptext;

		fpnode_data->bits = kset(fpnode_data->bits, chance);
		fpnode_data->data = node;

		memcpy(node, fpnode_slip,
		       sizeof(struct fpnode_type) * chance);

		fpnode_data = *fpdata;

		fpnode_data->bits = kset(fpnode_data->bits, change);
		fpnode_data->data = fpnode_lead;

		if (((spread + 3) & ~3) != ((change + 3) & ~3)) {
		    sfnote_text->link_m.mode
			(sfnote_text->link_m.data, &fpnode_data->data,
			 sizeof(struct fpnode_type) * ((change + 3) & ~3));

		    fpnode_slip = fpnode_data->data;
		    fpnode_slip += change - 1;

		    *fpslip = fpnode_slip;
		}

		*fpdata = fpnode_slip;

		fpnode_data = node;

		*fptext = fpnode_data;

		bits = fpnode_data->bits;

		fpnode_data->bits = kset(bits, 1);
		fpnode_data->data = NULL;

		length = Length(bits);
		if (sizeof(void *) < length) {
		    void *base;

		    status = sfnote_text->link_m.link
			(sfnote_text->link_m.data, &base, length);
		    if (status) {
			status = LINK_ERROR;

			fpnode_data->bits = nset(0, bits, 1);
		    } else {
			memcpy(base, fpnode_data->base, length);

			fpnode_data->base = base;
		    }
		}
	    }
	} else {
	    status = 0;

	    if (change & ~1) {
		fpnode_data = *fpdata;

		fpnode_data->bits = kset(fpnode_data->bits, change);
		fpnode_data->data = fpnode_lead;

		*fpdata = fpnode_slip;

		fpnode_data = *fptext;

		status = join_node(sfnote_text, fpnode_data, fpnode_slip);

		fpnode_data->bits = kset(fpnode_data->bits, 1);
		fpnode_data->data = NULL;
	    } else {
		fpnode_data = *fptext;

		fpnode_data->bits = kset(fpnode_data->bits, chance);
		fpnode_data->data = fpnode_lead;

		*fptext = fpnode_lead;

		fpnode_data = *fpdata;

		status = join_node(sfnote_text, fpnode_data, fpnode_lead);

		fpnode_data->bits = kset(fpnode_data->bits, 1);
		fpnode_data->data = NULL;
	    }
	}
    } else {
	void *root;

	status = sfnote_text->link_m.link
	    (sfnote_text->link_m.data, &root, sizeof(struct fpnode_type));
	if (status) {
	    status = LINK_ERROR;
	} else {
	    struct fpnode_type *fpnode_data;

	    fpnode_data = root;
	    *fptext = root;

	    fpnode_data->bits = eset(1);
	    fpnode_data->data = NULL;

	    sfnote_text->link_a.fpdeck.fpnode_data = fpnode_data;

	    status = join_node(sfnote_text, fpnode_data, *fpdata);
	}
    }

    return status;
}


static int
join_node(struct sfnote_type *sfnote_data, struct fpnode_type *fpnode_data,
	  struct fpnode_type *fpnode_pick)
{
    int status = 0;
    unsigned bond, debt;

    bond = fpnode_data->bits;
    debt = fpnode_pick->bits;

    if (true(Extant(debt))) {
	if (Prefix(debt)) {
	    if (Prefix(bond)) {
		unsigned ever, lead, size;

		ever = Length(debt);
		lead = Length(bond);

		size = lead + 1 + ever;
		if (sizeof(void *) < size) {
		    if (sizeof(void *) < lead) {
			void *line;

			line = fpnode_data->base;

			status = sfnote_data->link_m.mode
			    (sfnote_data->link_m.data, &line, size);
			if (status) {
			    status = MODE_ERROR;
			} else {
			    byte *base, *text;

			    base = line;
			    base += lead;
			    *base++ = Prefix(debt);
			    if (sizeof(void *) < ever) {
				text = fpnode_pick->base;
			    } else {
				text = (void *) &fpnode_pick->base;
			    }

			    memcpy(base, text, ever);

			    fpnode_data->base = line;
			    fpnode_data->bits = lset(size, bond);
			}
		    } else {
			void *line;

			status = sfnote_data->link_m.link
			    (sfnote_data->link_m.data, &line, size);
			if (status) {
			    status = LINK_ERROR;
			} else {
			    byte *base, *text;

			    base = line;
			    memcpy(base, &fpnode_data->base, lead);
			    base += lead;
			    *base++ = Prefix(debt);
			    if (sizeof(void *) < ever) {
				text = fpnode_pick->base;
			    } else {
				text = (void *) &fpnode_pick->base;
			    }

			    memcpy(base, text, ever);

			    fpnode_data->base = line;
			    fpnode_data->bits = lset(size, bond);
			}
		    }
		} else {
		    byte *base;

		    base = (void *) &fpnode_data->base;
		    base += lead;
		    *base++ = Prefix(debt);
		    memcpy(base, &fpnode_pick->base, ever);

		    fpnode_data->bits = lset(size, bond);
		}
	    } else {
		unsigned ever;

		ever = Length(debt);

		if (sizeof(void *) < ever) {
		    void *line;

		    status = sfnote_data->link_m.link
			(sfnote_data->link_m.data, &line, ever);
		    if (status) {
			status = LINK_ERROR;
		    } else {
			memcpy(line, fpnode_pick->base, ever);

			fpnode_data->base = line;
			fpnode_data->bits = pset(Prefix(debt), ever, bond);
		    }
		} else {
		    fpnode_data->base = fpnode_pick->base;
		    fpnode_data->bits = pset(Prefix(debt), ever, bond);
		}
	    }
	} else {
	}
    } else {
    }

    return status;
}


static int
pick_node(struct sfnote_type *sfnote_data, struct fpnode_type *fpnode_data,
	  struct fpnode_type *fpnode_pick)
{
    int status = 0;
    unsigned bond, debt;

    fpnode_data->data = fpnode_pick->data;

    bond = fpnode_data->bits;
    debt = fpnode_pick->bits;

    if (Extant(debt)) {
	if (Prefix(debt)) {
	    if (Prefix(bond)) {
		unsigned ever, lead, size;

		ever = Length(debt);
		lead = Length(bond);

		size = lead + 1 + ever;
		if (sizeof(void *) < size) {
		    if (sizeof(void *) < lead) {
			void *line;

			line = fpnode_data->base;

			status = sfnote_data->link_m.mode
			    (sfnote_data->link_m.data, &line, size);
			if (status) {
			    status = MODE_ERROR;
			} else {
			    byte *base, *text;

			    base = line;
			    base += lead;
			    *base++ = Prefix(debt);
			    if (sizeof(void *) < ever) {
				text = fpnode_pick->base;
			    } else {
				text = (void *) &fpnode_pick->base;
			    }

			    memcpy(base, text, ever);

			    fpnode_data->base = line;
			    fpnode_data->bits = nset(size, bond, Spread(debt));
			}
		    } else {
			void *line;

			status = sfnote_data->link_m.link
			    (sfnote_data->link_m.data, &line, size);
			if (status) {
			    status = LINK_ERROR;
			} else {
			    byte *base, *text;

			    base = line;
			    memcpy(base, &fpnode_data->base, lead);
			    base += lead;
			    *base++ = Prefix(debt);
			    if (sizeof(void *) < ever) {
				text = fpnode_pick->base;
			    } else {
				text = (void *) &fpnode_pick->base;
			    }

			    memcpy(base, text, ever);

			    fpnode_data->base = line;
			    fpnode_data->bits = nset(size, bond, Spread(debt));
			}
		    }

		    if (sizeof(void *) < ever) {
			int excess;

			excess = sfnote_data->link_m.free
			    (sfnote_data->link_m.data, fpnode_pick->base);
			if (excess) {
			    if (status) {
			    } else {
				status = FREE_ERROR;
			    }
			}
		    }
		} else {
		    byte *base;

		    base = (void *) &fpnode_data->base;
		    base += lead;
		    *base++ = Prefix(debt);
		    memcpy(base, &fpnode_pick->base, ever);

		    fpnode_data->bits = nset(size, bond, Spread(debt));
		}
	    } else {
		fpnode_data->bits = debt;
		fpnode_data->base = fpnode_pick->base;
	    }
	} else {
	    fpnode_data->bits = kset(bond, Spread(debt));
	}
    } else {
	int e;

	if (sizeof(void *) < Length(bond)) {
	    status = sfnote_data->link_m.free
		(sfnote_data->link_m.data, fpnode_data->base);
	    if (status) {
		status = FREE_ERROR;
	    }
	}

	e = Prefix(bond);

	fpnode_data->bits = e ? dset(Prefix(bond), Length(debt)) : debt;
	fpnode_data->base = fpnode_pick->base;
    }

    return status;
}


static int
tear_note(struct sfnote_type *sfnote_data, const char *name, int clear,
	  struct sfnote_type *sfnote_text)
{
    const char *lead;
    int status;
    struct fpnode_type *fpdata, *fpnode_data, *fpnode_slip = NULL, *fptext;

    fpdata = sfnote_data->link_a.fpdeck.fpnode_data;
    fptext = sfnote_text->link_a.fpdeck.fpnode_data;

    fpnode_data = fpdata;

    lead = name;

    do {
	byte *slip;
	int e;
	unsigned length;

	e = prefix(fpnode_data);
	if (e) {
	    int c;

	    c = *fast(name);
	    if (e ^ c) {
		if (c < e) {
		    sfnote_data->link_a.fpdeck.fpnode_data = NULL;
		    sfnote_text->link_a.fpdeck.fpnode_data = fpnode_data;
		} else {
		}

		status = 0;

		break;
	    } else {
		name++;
		advance_node(fpnode_data, slip, length);
	    }
	} else {
	    length = 0;
	    slip = NULL;
	}

	while (1) {
	    int c;

	    c = *fast(name);
	    if (c) {
		name++;
		if (length) {
		    e = *slip;
		    if (c == e) {
			length--;
			slip++;
		    } else {
			if (c < e) {
			    if (fpnode_slip) {
				status = copy_pass
				    (sfnote_text, fpdata, fptext, fpnode_data,
				     0, fpnode_slip);
			    } else {
				sfnote_data->link_a.fpdeck.fpnode_data = NULL;
				sfnote_text->link_a.fpdeck.fpnode_data =
				    fpnode_data;

				status = 0;
			    }
			} else {
			    if (fpnode_slip) {
				status = copy_pass
				    (sfnote_text, fpdata, fptext, fpnode_data,
				     1, fpnode_slip);
			    } else {
				status = 0;
			    }
			}

			break;
		    }
		} else {
		    unsigned bits;

		    bits = fpnode_data->bits;
		    if (Extant(bits)) {
			struct fpnode_type *fpnode_lead, *fpnode_wrap;
			unsigned effect;

			fpnode_wrap = fpnode_slip;

			effect = Effect(bits);

			status = copy_text
			    (sfnote_text, &fpdata, &fptext, &fpnode_data,
			     fpnode_wrap);
			if (status) {
			    break;
			}

			fpnode_slip = fpnode_data;

			fpnode_lead = fpnode_data->data;
			fpnode_data = fpnode_lead;

			Progress_node(fpnode_data, effect);

			e = prefix(fpnode_data);
			if (e ^ c) {
			    if (e < c) {
				status = copy_pass
				    (sfnote_text, fpdata, fptext, fpnode_data,
				     1, fpnode_slip);
			    } else {
				status = copy_pass
				    (sfnote_text, fpdata, fptext, fpnode_data,
				     0, fpnode_slip);
			    }

			    break;
			} else {
			    advance_node(fpnode_data, slip, length);
			}
		    } else {
			if (fpnode_slip) {
			    status = copy_pass
				(sfnote_text, fpdata, fptext, fpnode_data, 1,
				 fpnode_slip);
			} else {
			    status = 0;
			}

			break;
		    }
		}
	    } else {
		if (fpnode_slip) {
		    do {
			if (length | !clear) {
			    clear = 0;
			} else {
			    if (extant(fpnode_data)) {
				struct fpnode_type *fpnode_lead;

				fpnode_lead = fpnode_data->data;
				if (prefix(fpnode_lead)) {
				    clear = 0;
				} else {
				    status = copy_text
					(sfnote_text, &fpdata, &fptext,
					 &fpnode_data, fpnode_slip);
				    if (status) {
					break;
				    }

				    fpnode_slip = fpnode_data;

				    fpnode_data = fpnode_lead;
				}
			    }
			}

			status = copy_pass
			    (sfnote_text, fpdata, fptext, fpnode_data, clear,
			     fpnode_slip);
		    } while (0);
		} else {
		    status = 0;

		    do {
			if (length | !clear) {
			} else {
			    if (extant(fpnode_data)) {
				struct fpnode_type *fpnode_lead;

				fpnode_lead = fpnode_data->data;
				if (prefix(fpnode_lead)) {
				} else {
				    status = copy_text
					(sfnote_text, &fpdata, &fptext,
					 &fpnode_data, fpnode_slip);
				    if (status) {
					break;
				    }

				    fpnode_slip = fpnode_data;

				    fpnode_data = fpnode_lead;

				    status = copy_pass
					(sfnote_text, fpdata, fptext,
					 fpnode_data, clear, fpnode_slip);

				    break;
				}
			    } else {
				break;
			    }
			}

			sfnote_data->link_a.fpdeck.fpnode_data = NULL;
			sfnote_text->link_a.fpdeck.fpnode_data = fpnode_data;
		    } while (0);
		}

		break;
	    }
	}
    } while (0);

    return status;
}


static int
wrap_ever(struct sfnote_type *sfnote_data)
{
    int status = 0;
    struct fpnode_type *fpnode_data;

    fpnode_data = sfnote_data->link_a.fpdeck.fpnode_data;
    if (fpnode_data) {
	unsigned bits, effect;

	bits = fpnode_data->bits;
	if (Extant(bits)) {
	    struct fpnode_type *fpnode_lead;

	    fpnode_lead = fpnode_data->data;
	    if (fpnode_lead) {
		do {
		    effect = Effect(bits);

		    fpnode_lead = fpnode_data->data;

		    if (extant(fpnode_lead)) {
			if (fpnode_lead->data) {
			} else {
			    if (sizeof(void *) < length(fpnode_lead)) {
				status = sfnote_data->link_m.free
				    (sfnote_data->link_m.data,
				     fpnode_lead->base);
				if (status) {
				    status = FREE_ERROR;
				}
			    }

			    memmove
				(fpnode_lead, fpnode_lead + 1,
				 sizeof(struct fpnode_type) * effect);

			    fpnode_data->bits = iset(bits, effect);

			    if (effect == 1) {
				int excess;

				excess = pick_node
				    (sfnote_data, fpnode_data, fpnode_lead);
				if (excess) {
				    if (status) {
				    } else {
					status = FREE_ERROR;
				    }
				}

				excess = sfnote_data->link_m.free
				    (sfnote_data->link_m.data, fpnode_lead);
				if (excess) {
				    if (status) {
				    } else {
					status = FREE_ERROR;
				    }
				}
			    } else {
				if (!(effect & 3)) {
				    status = sfnote_data->link_m.mode
					(sfnote_data->link_m.data,
					 &fpnode_data->data,
					 sizeof(struct fpnode_type) * effect);
				    if (status) {
					status = MODE_ERROR;
				    }
				}
			    }

			    if (1) {
				break;
			    }
			}
		    }

		    fpnode_data = fpnode_lead;

		    bits = fpnode_data->bits;
		} while (Extant(bits));
	    } else {
		int excess;

		sfnote_data->link_a.fpdeck.fpnode_data = NULL;

		if (sizeof(void *) < Length(bits)) {
		    status = sfnote_data->link_m.free
			(sfnote_data->link_m.data, fpnode_data->base);
		    if (status) {
			status = FREE_ERROR;
		    }
		}

		excess = sfnote_data->link_m.free
		    (sfnote_data->link_m.data, fpnode_data);
		if (excess) {
		    if (status) {
		    } else {
			status = FREE_ERROR;
		    }
		}
	    }
	} else {
	    fpnode_data->bits = dset(*fpnode_data->base, Length(bits));
	}
    }

    return status;

}


static int
wrap_lead(struct sfnote_type *sfnote_data)
{
    int status = 0;
    struct fpnode_type *fpnode_data;

    fpnode_data = sfnote_data->link_a.fpdeck.fpnode_data;
    if (fpnode_data) {
	unsigned bits, effect;

	bits = fpnode_data->bits;
	if (Extant(bits)) {
	    struct fpnode_type *fpnode_lead;

	    fpnode_lead = fpnode_data->data;
	    if (fpnode_lead) {
		do {
		    effect = Effect(bits);

		    fpnode_lead = fpnode_data->data;

		    fpnode_lead += effect;

		    if (extant(fpnode_lead)) {
			if (fpnode_lead->data) {
			} else {
			    if (sizeof(void *) < length(fpnode_lead)) {
				status = sfnote_data->link_m.free
				    (sfnote_data->link_m.data,
				     fpnode_lead->base);
				if (status) {
				    status = FREE_ERROR;
				}
			    }

			    fpnode_data->bits = iset(bits, effect);

			    if (effect == 1) {
				int excess;

				fpnode_lead--;

				excess = pick_node
				    (sfnote_data, fpnode_data, fpnode_lead);
				if (excess) {
				    if (status) {
				    } else {
					status = FREE_ERROR;
				    }
				}

				excess = sfnote_data->link_m.free
				    (sfnote_data->link_m.data, fpnode_lead);
				if (excess) {
				    if (status) {
				    } else {
					status = FREE_ERROR;
				    }
				}
			    } else {
				if (!(effect & 3)) {
				    status = sfnote_data->link_m.mode
					(sfnote_data->link_m.data,
					 &fpnode_data->data,
					 sizeof(struct fpnode_type) * effect);
				    if (status) {
					status = MODE_ERROR;
				    }
				}
			    }

			    if (1) {
				break;
			    }
			}
		    }

		    fpnode_data = fpnode_lead;

		    bits = fpnode_data->bits;
		} while (Extant(bits));
	    } else {
		int excess;

		sfnote_data->link_a.fpdeck.fpnode_data = NULL;

		if (sizeof(void *) < Length(bits)) {
		    status = sfnote_data->link_m.free
			(sfnote_data->link_m.data, fpnode_data->base);
		    if (status) {
			status = FREE_ERROR;
		    }
		}

		excess = sfnote_data->link_m.free
		    (sfnote_data->link_m.data, fpnode_data);
		if (excess) {
		    if (status) {
		    } else {
			status = FREE_ERROR;
		    }
		}
	    }
	} else {
	    fpnode_data->bits = dset(*fpnode_data->base, Length(bits));
	}
    }

    return status;
}


int
x1f4_tear_sfnote(void *sfdata, const char *name, int clear, void *sftext)
{
    int status;

    if (sfnote(sftext)->link_a.fpdeck.fpnode_data) {
	status = SIZE_ERROR;
    } else {
	if (sfnote(sfdata)->link_a.fpdeck.fpnode_data) {
	    int excess;

	    status = tear_note(sfdata, name, clear ? 1 : 0, sftext);

	    excess = wrap_lead(sfdata);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }

	    excess = wrap_ever(sftext);
	    if (excess) {
		if (status) {
		} else {
		    status = excess;
		}
	    }
	} else {
	    status = 0;
	}
    }

    return status;
}
